from datetime import datetime
import os
import json

import logging
from pathlib import Path
import pkgutil
import re
import time
import uuid

from .bundle_loader import BundleLoader

from .. import config
from tqdm import tqdm
from ..services.arangodb_service import ArangoDBService
from jsonschema import validate
from arango.collection import StandardCollection
import arango.exceptions


from .. import utils

module_logger = logging.getLogger("data_ingestion_service")
SMO_TYPES = ["marking-definition", "extension-definition", "language-content"]
LARGE_FILE_SIZE = 80 * 1024 * 1024


class Stix2Arango:
    EMBEDDED_RELATIONSHIP_RE = re.compile(r"([a-z\-_]+)[_\-]refs{0,1}")
    filename = "bundle.json"
    ARANGODB_URL = f"http://{config.ARANGODB_HOST}:{config.ARANGODB_PORT}"

    def __init__(
        self,
        database,
        collection,
        file,
        create_collection=True,
        create_db=True,
        stix2arango_note="",
        ignore_embedded_relationships=False,
        ignore_embedded_relationships_sro=True,
        ignore_embedded_relationships_smo=True,
        include_embedded_relationships_attributes=None,
        bundle_id=None,
        username=config.ARANGODB_USERNAME,
        password=config.ARANGODB_PASSWORD,
        host_url=ARANGODB_URL,
        is_large_file=False,
        skip_default_indexes=False,
        create_taxii_views=True,
        **kwargs,
    ):
        """
        `modify_fn` should modify in-place, returned value is discarded
        """

        self.alter_functions = []

        self.core_collection_vertex, self.core_collection_edge = (
            utils.get_vertex_and_edge_collection_names(collection)
        )
        EDGE_COLLECTIONS = [self.core_collection_edge]
        VERTEX_COLLECTIONS = [self.core_collection_vertex]

        self.arango = ArangoDBService(
            database,
            VERTEX_COLLECTIONS,
            EDGE_COLLECTIONS,
            create=create_collection,
            create_db=create_db,
            username=username,
            password=password,
            host_url=host_url,
            **kwargs,
        )

        self.arangodb_extra_data = {}

        self.file = file
        self._is_large_file = is_large_file
        self.note = stix2arango_note or ""
        self.identity_ref = utils.load_file_from_url(config.STIX2ARANGO_IDENTITY).copy()
        self.default_ref_objects = [
            utils.load_file_from_url(link).copy()
            for link in config.MARKING_DEFINITION_REFS + config.IDENTITY_REFS
        ]
        self.bundle_id = bundle_id
        self.ignore_embedded_relationships = ignore_embedded_relationships
        self.ignore_embedded_relationships_smo = ignore_embedded_relationships_smo
        self.ignore_embedded_relationships_sro = ignore_embedded_relationships_sro
        self.include_embedded_relationships_attributes = include_embedded_relationships_attributes
        self.object_key_mapping = {}
        if create_collection:
            self.create_s2a_indexes()
            if not skip_default_indexes:
                self.create_default_indexes()
            if create_taxii_views:
                self.create_taxii_views()

        if self.file:
            self.filename = Path(self.file).name

    @property
    def is_large_file(self):
        return self._is_large_file or os.path.getsize(self.file) > LARGE_FILE_SIZE

    def alter_objects(self, objects: list[dict]):
        for obj in objects:
            obj.update(self.arangodb_extra_data)
            for fn in self.alter_functions:
                try:
                    fn(obj)
                except Exception as e:
                    logging.warning(f"alter function {fn} failed on {obj}")
                    logging.warning(
                        f"alter function {fn} failed on {obj}", exc_info=True
                    )

    def add_object_alter_fn(self, modify_fn):
        if not callable(modify_fn):
            raise ValueError("Bad modification function passed")
        self.alter_functions.append(modify_fn)

    def create_s2a_indexes(self):
        for name, collection in self.arango.collections.items():
            collection.add_index(
                dict(
                    type="persistent",
                    name="s2a_search",
                    sparse=False,
                    fields=[
                        "id",
                        "modified",
                        "_is_latest",
                    ],
                    inBackground=True,
                    storedValues=["_record_modified", "_key", "_id"],
                )
            )

            collection.add_index(
                dict(
                    type="persistent",
                    name="s2a_unique_constraint",
                    unique=True,
                    fields=[
                        "id",
                        "_record_md5_hash",
                    ],
                    inBackground=True,
                )
            )
            if name.endswith("_edge_collection"):
                collection.add_index(
                    dict(
                        type="persistent",
                        name="s2a_search_edge",
                        sparse=True,
                        fields=[
                            "_from",
                            "_is_latest",
                        ],
                        inBackground=True,
                        storedValues=["_id"],
                    )
                )

    def create_analyzer(self, *args, **kwargs):
        try:
            return self.arango.db.create_analyzer(*args, **kwargs)
        except arango.exceptions.AnalyzerCreateError as e:
            if e.error_code != 10:
                raise
    
    def create_taxii_views(self):
        views = set()
        self.create_analyzer(
            name="date_transform",
            analyzer_type="aql",
            features=[],
            properties={
                "queryString": "RETURN DATE_TIMESTAMP(@param)*1000 + TO_NUMBER(LAST(REGEX_MATCHES(@param, '.+\\\\.(\\\\d{6}).*')))%1000",
                "collapsePositions": False,
                "keepNull": True,
                "batchSize": 1000,
                "memoryLimit": 10485760,
                "returnType": "number",
            },
        )
        for name, collection in self.arango.collections.items():
            logging.info(f'creating taxii index for {name}')
            collection.add_index(
                dict(
                    type="inverted",
                    name="taxii_search",
                    sparse=True,
                    fields=[
                        dict(name="_record_created", analyzer="date_transform"),
                        "modified",
                        "id",
                        "_taxii.visible",
                        "_taxii.last",
                        "_taxii.first",
                        "spec_version",
                        "type",
                    ],
                    inBackground=True,
                    storedValues=["_key", "_created"],
                    primarySort={
                        "fields": [{"field": "_record_created", "direction": "asc"}]
                    },
                )
            )
            views.add(
                "ats__"
                + name.removesuffix("_vertex_collection").removesuffix(
                    "_edge_collection"
                )
            )

    def create_default_indexes(self):
        for name, collection in self.arango.collections.items():
            module_logger.info(
                f"creating indexes for collection {collection.db_name}/{name}"
            )
            time = int(datetime.now().timestamp())

            collection.add_index(
                dict(
                    type="persistent",
                    fields=["id"],
                    storedValues=[
                        "modified",
                        "created",
                        "type",
                        "_record_modified",
                        "spec_version",
                        "_record_md5_hash",
                    ],
                    inBackground=True,
                    name=f"by_stix_id_{time}",
                )
            )
            collection.add_index(
                dict(
                    type="persistent",
                    fields=["id", "type"],
                    storedValues=[
                        "modified",
                        "created",
                        "_record_modified",
                        "spec_version",
                        "_record_md5_hash",
                    ],
                    inBackground=True,
                    name=f"by_stix_id_type_{time}",
                )
            )
            collection.add_index(
                dict(
                    type="persistent",
                    fields=["modified", "created"],
                    storedValues=[
                        "type",
                        "_record_modified",
                        "id",
                        "spec_version",
                        "_record_md5_hash",
                    ],
                    inBackground=True,
                    name=f"by_stix_version_{time}",
                )
            )
            collection.add_index(
                dict(
                    type="persistent",
                    fields=["type"],
                    storedValues=[
                        "modified",
                        "created",
                        "_record_modified",
                        "id",
                        "spec_version",
                        "_record_md5_hash",
                    ],
                    inBackground=True,
                    name=f"by_stix_type_{time}",
                )
            )
            collection.add_index(
                dict(
                    type="persistent",
                    fields=["_record_modified", "_record_created"],
                    storedValues=[
                        "modified",
                        "created",
                        "type",
                        "id",
                        "spec_version",
                        "_record_md5_hash",
                    ],
                    inBackground=True,
                    name=f"by_insertion_time_{time}",
                )
            )
            if name.endswith("_edge_collection"):
                collection.add_index(
                    dict(
                        type="persistent",
                        fields=["source_ref", "target_ref", "relationship_type"],
                        storedValues=[
                            "modified",
                            "created",
                            "type",
                            "_record_modified",
                            "spec_version",
                            "_record_md5_hash",
                            "id",
                        ],
                        inBackground=True,
                        name=f"relation_from_{time}",
                    )
                )
                collection.add_index(
                    dict(
                        type="persistent",
                        fields=["target_ref", "source_ref", "relationship_type"],
                        storedValues=[
                            "modified",
                            "created",
                            "type",
                            "_record_modified",
                            "spec_version",
                            "_record_md5_hash",
                            "id",
                        ],
                        inBackground=True,
                        name=f"relation_to_{time}",
                    )
                )
                collection.add_index(
                    dict(
                        type="persistent",
                        fields=["relationship_type", "target_ref", "source_ref"],
                        storedValues=[
                            "modified",
                            "created",
                            "type",
                            "_record_modified",
                            "spec_version",
                            "_record_md5_hash",
                            "id",
                        ],
                        inBackground=True,
                        name=f"relation_type_{time}",
                    )
                )

    def default_objects(self):
        object_list = self.default_ref_objects
        for obj in json.loads(
            pkgutil.get_data("stix2arango", "templates/marking-definition.json")
        ):
            object_list.append(obj)
        return object_list

    def process_bundle_into_graph(
        self, objects_in, notes=None, is_default_objects=False
    ):
        module_logger.info(f"Reading vertex from file {self.file} now")

        objects = []
        insert_data = []  # That would be the overall statement
        for obj in tqdm(objects_in, desc="upload_vertices"):
            if obj.get("type") == "relationship":
                continue
            obj.setdefault("_stix2arango_note", notes or self.note)
            obj["_record_md5_hash"] = utils.generate_md5(obj)
            if not is_default_objects:
                obj["_bundle_id"] = self.bundle_id or ""
                obj["_file_name"] = self.filename or ""
                obj.update(self.arangodb_extra_data)
            objects.append(obj)
            insert_data.append(
                [obj.get("type"), obj.get("id"), True if "modified" in obj else False]
            )

        module_logger.info(
            f"Inserting objects into database. Total objects: {len(objects)}"
        )
        with self.arango.transactional(
            exclusive=[self.core_collection_edge, self.core_collection_vertex]
        ):
            inserted_object_ids, existing_objects = (
                self.arango.insert_several_objects_chunked(
                    objects, self.core_collection_vertex
                )
            )
            deprecated_key_ids = self.arango.update_is_latest_several_chunked(
                inserted_object_ids,
                self.core_collection_vertex,
                self.core_collection_edge,
            )

        self.update_object_key_mapping(
            self.core_collection_vertex, objects, existing_objects
        )
        return inserted_object_ids, existing_objects, deprecated_key_ids

    def update_object_key_mapping(self, collection, objects, existing_objects={}):
        for obj in objects:
            db_key = existing_objects.get(f"{obj['id']};{obj['_record_md5_hash']}")
            if not db_key and obj.get("_key"):
                db_key = f"{collection}/{obj['_key']}"
            if db_key:
                self.object_key_mapping[obj["id"]] = db_key

    def map_relationships(self, filename, objects_in):

        module_logger.info("Mapping Prebuilt Relationship Objects -> ")
        objects = []
        inserted_data = []
        obj: dict
        for obj in tqdm(objects_in, desc="upload_edges"):
            if obj.get("type") == "relationship":

                source_ref = obj.get("source_ref")
                target_ref = obj.get("target_ref")

                obj.setdefault("_from", f"{self.core_collection_vertex}/{source_ref}")
                obj.setdefault("_to", f"{self.core_collection_vertex}/{target_ref}")
                obj["_bundle_id"] = self.bundle_id or ""
                obj["_file_name"] = filename
                obj.setdefault("_is_ref", False)
                obj.setdefault("_stix2arango_note", self.note)
                # obj['_record_md5_hash'] = utils.generate_md5(obj)
                obj.update(self.arangodb_extra_data)
                objects.append(obj)
                inserted_data.append(
                    [
                        obj.get("type"),
                        obj.get("id"),
                        True if "modified" in obj else False,
                    ]
                )

        module_logger.info(
            f"Inserting relationship into database. Total objects: {len(objects)}"
        )
        with self.arango.transactional(
            exclusive=[self.core_collection_edge, self.core_collection_vertex]
        ):
            inserted_object_ids, existing_objects = (
                self.arango.insert_relationships_chunked(
                    objects, self.object_key_mapping, self.core_collection_edge
                )
            )
            deprecated_key_ids = self.arango.update_is_latest_several_chunked(
                inserted_object_ids,
                self.core_collection_edge,
                self.core_collection_edge,
            )
        self.update_object_key_mapping(
            self.core_collection_edge, objects, existing_objects
        )
        return inserted_object_ids, deprecated_key_ids

    def map_embedded_relationships(self, bundle_objects, inserted_object_ids):
        objects = []
        for obj in tqdm(bundle_objects, desc="upload_embedded_edges"):
            if obj["id"] not in inserted_object_ids:
                continue
            if self.include_embedded_relationships_attributes:
                pass
            elif (
                self.ignore_embedded_relationships_smo and obj["type"] in SMO_TYPES
            ) or (
                self.ignore_embedded_relationships_sro and obj["type"] == "relationship"
            ):
                continue

            for ref_type, targets in utils.get_embedded_refs(obj, attributes=self.include_embedded_relationships_attributes):
                utils.create_relationship_obj(
                    obj=obj,
                    source=obj.get("id"),
                    targets=targets,
                    relationship=ref_type,
                    arango_obj=self,
                    bundle_id=self.bundle_id or "",
                    insert_statement=objects,
                    extra_data=self.arangodb_extra_data,
                )

        module_logger.info(
            f"Inserting embedded relationship into database. Total objects: {len(objects)}"
        )

        self.alter_objects(objects)
        inserted_object_ids = []
        existing_objects = {}
        for chunk in utils.chunked(objects, 20_000):
            with self.arango.transactional(
                exclusive=[self.core_collection_edge, self.core_collection_vertex]
            ):
                inserted, existing = self.arango.insert_relationships_chunked(
                    chunk, self.object_key_mapping, self.core_collection_edge
                )
                inserted_object_ids.extend(inserted)
                existing_objects.update(existing)
        self.arango.update_is_latest_several_chunked(
            inserted_object_ids, self.core_collection_edge, self.core_collection_edge
        )
        return inserted_object_ids, existing_objects

    def import_default_objects(self):
        self.process_bundle_into_graph(
            self.default_objects(),
            notes="automatically imported on collection creation",
            is_default_objects=True,
        )

    def run(self, data=None):
        if not data and not self.file:
            raise Exception("file or data must be passed")

        if not data:
            if self.is_large_file:
                module_logger.info("using large file mode")
                os.makedirs("db_loader_tempfiles", exist_ok=True)
                bundle_loader = BundleLoader(
                    self.file, db_path=f"db_loader_tempfiles/mydb_{time.time()}.sqlite"
                )
                for chunk in bundle_loader.chunks:
                    self.run_with_bundle(
                        {
                            "type": "bundle",
                            "objects": chunk,
                            "id": bundle_loader.bundle_id,
                        }
                    )
                return
            else:
                module_logger.info("using regular file mode")
                with open(self.file, "r") as input_file:
                    file_data = input_file.read()
                    try:
                        data = json.loads(file_data)
                        self.bundle_id = self.bundle_id or data["id"]
                    except Exception as e:
                        raise Exception("Invalid file type")
                try:
                    validate(instance=data, schema=config.json_schema)
                except Exception as e:
                    raise Exception("Invalid File structure")

        self.run_with_bundle(data)

    def run_with_bundle(self, bundle_dict):
        if bundle_dict.get("type", None) != "bundle":
            raise Exception("Provided file is not a STIX bundle. Aborted")

        all_objects = bundle_dict["objects"]
        self.alter_objects(all_objects)

        module_logger.info(
            f"Loading default objects from url and store into {self.core_collection_vertex}"
        )
        self.import_default_objects()

        module_logger.info(
            f"Load objects from file: {self.file} and store into {self.core_collection_vertex}"
        )
        inserted_object_ids, _, deprecated_key_ids1 = self.process_bundle_into_graph(
            all_objects
        )
        module_logger.info("Mapping relationships now -> ")
        inserted_relationship_ids, deprecated_key_ids2 = self.map_relationships(
            self.filename, all_objects
        )

        if (not self.ignore_embedded_relationships) or self.include_embedded_relationships_attributes:
            module_logger.info(
                "Creating new embedded relationships using _refs and _ref"
            )
            self.map_embedded_relationships(
                all_objects, inserted_object_ids + inserted_relationship_ids
            )

        with self.arango.transactional(
            exclusive=[self.core_collection_edge, self.core_collection_vertex]
        ):
            self.arango.deprecate_relationships(
                deprecated_key_ids1, self.core_collection_edge
            )
            self.arango.deprecate_relationships(
                deprecated_key_ids2, self.core_collection_edge
            )



import os
from dotenv import load_dotenv

import arango
import arango.exceptions
from arango.database import StandardDatabase
from arango import ArangoClient

from stix2arango.stix2arango.stix2arango import Stix2Arango

## run this like `ARANGODB_DATABASE=vulmatch_database python add_ref_types.py` if your database is created by any s2a version < 0.0.4

load_dotenv()

host_url = os.getenv('ARANGODB_HOST_URL')
client = ArangoClient(host_url, request_timeout=None)
PASSWORD = os.getenv('ARANGODB_PASSWORD')
db = client.db(username=os.getenv('ARANGODB_USERNAME'), password=PASSWORD, name=os.getenv('ARANGODB_DATABASE'))
collections = []
for collection in db.collections():
    collection_name: str = collection['name']
    if not (collection_name.endswith('_vertex_collection') or collection_name.endswith('_edge_collection')):
        continue
    c = db.collection(collection_name)
    indexes = {index['name']: index for index in c.indexes()}
    index = indexes.get('taxii_search')
    if index:
        c.delete_index('taxii_search')
    collections.append([c.count(), collection_name, c])

collections = dict(v[1:] for v in sorted(collections, key=lambda x: x[0]))

s2a = Stix2Arango(db.db_name, "", "", username=db.username, host_url=host_url, password=PASSWORD, create_collection=False, create_db=False)
s2a.arango.collections = collections
s2a.create_taxii_views()


from contextlib import contextmanager
from functools import lru_cache
import re
import requests
import logging
import uuid
import json
import hashlib
import os
from . import config
from datetime import UTC, datetime

module_logger = logging.getLogger("data_ingestion_service")
from arango.database import StandardDatabase


EMBEDDED_RELATIONSHIP_RE = re.compile(r"([a-z\-_]+)[_\-]refs{0,1}")


@lru_cache
def load_file_from_url(url):
    try:
        response = requests.get(url)
        response.raise_for_status()  # Raise an HTTPError for bad responses
        return json.loads(response.text)
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error loading JSON from {url}: {e}") from e


def create_relationship_obj(
    obj: dict,
    source: str,
    targets: list,
    relationship: str,
    insert_statement,
    bundle_id: str,
    arango_obj=None,
    extra_data=None,
):
    if not isinstance(targets, list):
        return []
    for target in targets:
        relationship_object = {
            "relationship_type": relationship,
        }
        for key in ["created", "modified", "object_marking_refs"]:
            if key in obj:
                relationship_object[key] = obj[key]

        relationship_object.update(
            id="relationship--"
            + str(
                uuid.uuid5(
                    config.namespace,
                    "{}+{}+{}".format(relationship, source, target),
                )
            ),
            created_by_ref=arango_obj.identity_ref["id"],
        )

        if not isinstance(target, str):
            continue
        relationship_object["source_ref"] = source
        relationship_object["target_ref"] = target
        relationship_object["_from"] = f"{arango_obj.core_collection_vertex}/{source}"
        relationship_object["_to"] = f"{arango_obj.core_collection_vertex}/{target}"
        relationship_object["_bundle_id"] = bundle_id
        relationship_object["_file_name"] = os.path.basename(arango_obj.file or "")
        relationship_object["_stix2arango_note"] = arango_obj.note
        relationship_object["_is_ref"] = True
        relationship_object["type"] = "relationship"
        relationship_object["spec_version"] = "2.1"
        relationship_object["external_references"] = [
            {
                "source_name": "stix2arango",
                "description": "embedded-relationship",
            }
        ]
        if extra_data:
            relationship_object.update(extra_data)

        insert_statement.append(relationship_object)
    return


def generate_md5(obj: dict):
    obj_copy = {k: v for k, v in obj.items() if not k.startswith("_")}
    for k in ["_from", "_to", "_stix2arango_note"]:
        if v := obj.get(k):
            obj_copy[k] = v
    json_str = json.dumps(obj_copy, sort_keys=True, default=str).encode("utf-8")
    return hashlib.md5(json_str).hexdigest()


def get_vertex_and_edge_collection_names(name):
    ENDINGS = ["vertex_collection", "edge_collection"]
    splits = name.split("_")
    if "_".join(splits[-2:]) in ENDINGS:
        splits = splits[:-2]
    return "_".join(splits + [ENDINGS[0]]), "_".join(splits + [ENDINGS[1]])


def chunked(iterable, n):
    if not iterable:
        return []
    for i in range(0, len(iterable), n):
        yield iterable[i : i + n]


def remove_duplicates(objects):
    objects_hashmap = {}
    for object in objects:
        md5hash = object.get("_record_md5_hash")
        if not md5hash:
            md5hash = generate_md5(object)
        key = "{id}/{hash}".format(id=object["id"], hash=md5hash)
        objects_hashmap[key] = object
    return list(objects_hashmap.values())


def get_embedded_refs(object: list | dict, xpath: list = [], attributes=None):
    embedded_refs = []
    if isinstance(object, dict):
        for key, value in object.items():
            if key in ["source_ref", "target_ref"]:
                continue
            if match := EMBEDDED_RELATIONSHIP_RE.fullmatch(key):
                relationship_type = "-".join(xpath + match.group(1).split("_"))
                targets = value if isinstance(value, list) else [value]
                targets = [
                    _target
                    for _target in targets
                    if _target and isinstance(_target, str)
                ]
                if not targets:
                    continue
                if attributes and key not in attributes:
                    continue
                embedded_refs.append((relationship_type, targets))
            elif isinstance(value, list):
                embedded_refs.extend(
                    get_embedded_refs(value, xpath + [key], attributes=attributes)
                )
    elif isinstance(object, list):
        for obj in object:
            if isinstance(obj, dict):
                embedded_refs.extend(
                    get_embedded_refs(obj, xpath, attributes=attributes)
                )
    return embedded_refs



# python3 -m unittest tests/test_22-embedded-sro-update-for-smo.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test22"
        cls.FILES = [
            dict(file="smo-embedded-ref-1.json", ignore_embedded_relationships=False, stix2arango_note="test22"),
            dict(file="smo-embedded-ref-2.json", ignore_embedded_relationships=False, stix2arango_note="test22"),
        ]


    def test_query_1(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test22_edge_collection
            FILTER doc._is_latest == true
            AND doc._is_ref == true
            AND doc._stix2arango_note != "automatically imported on collection creation"
              RETURN doc
        )
        """
        expected_result = [3]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the latest object has 3 refs

    def test_query_2(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test22_edge_collection
            FILTER doc._is_latest == false
            AND doc._is_ref == true
            AND doc._stix2arango_note != "automatically imported on collection creation"
              RETURN doc
        )
        """
        expected_result = [3]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # whilst the smo is updated with different md5 hash, the embedded sros created have exactly the same md5 hash, so it is not updated

    def test_query_3(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test22_edge_collection
            FILTER doc.id == "relationship--1cce4dca-d80c-5650-a9c2-858ad6707779"
            AND NOT HAS(doc, 'modified') AND HAS(doc, 'created') AND doc._is_latest
            RETURN doc
        )
        """
        expected_result = [1]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # check created time exists

    def test_query_4(self):
        query = """
        FOR doc IN test22_edge_collection
            FILTER doc.id == "relationship--1cce4dca-d80c-5650-a9c2-858ad6707779" AND doc._is_latest
            RETURN {
                modified: doc.modified,
                created: doc.created
            }
        """
        expected_result = [
          {
            "modified": None,
            "created": "2016-08-01T00:00:00.000Z"
          }
        ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # follow up to test 3 checking that created property is as expected

if __name__ == '__main__':
    unittest.main()



# python3 -m unittest tests/test_20-embedded-sro-update-for-sdo.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test20"
        cls.FILES = [
            dict(file="embedded-ref-object.json", ignore_embedded_relationships=False, stix2arango_note="test20"),
            dict(file="embedded-ref-object-updated.json", ignore_embedded_relationships=False, stix2arango_note="test20"),
        ]


    def test_query_1(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test20_edge_collection
            FILTER doc._is_latest == true
            AND doc._is_ref == true
            AND doc._stix2arango_note != "automatically imported on collection creation"
              RETURN doc
        )
        """
        expected_result = [4]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the latest object has ref properties

    def test_query_2(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test20_edge_collection
            FILTER doc._is_latest == false
            AND doc._is_ref == true
            AND doc._stix2arango_note != "automatically imported on collection creation"
              RETURN doc
        )
        """
        expected_result = [3]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the old object had 3 ref properties

    def test_query_3(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test20_edge_collection
            FILTER doc.id == "relationship--5b32a703-4317-5f58-b1ce-03735c756035"
            AND HAS(doc, 'modified') AND HAS(doc, 'created')
            RETURN doc
        )
        """
        expected_result = [2]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the embedded SRO should have modified and created time because the source indicator object does

    def test_query_4(self):
        query = """
        FOR doc IN test20_edge_collection
            FILTER doc.id == "relationship--5b32a703-4317-5f58-b1ce-03735c756035"
            SORT doc.modified
            RETURN {
                modified: doc.modified,
                created: doc.created
            }
        """
        expected_result = [
          {
            "modified": "2023-02-14T00:00:00.000Z",
            "created": "2021-12-07T00:00:00.000Z"
          },
          {
            "modified": "2024-02-14T00:00:00.000Z",
            "created": "2021-12-07T00:00:00.000Z"
          }
        ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # check modified times are correct

if __name__ == '__main__':
    unittest.main()


# python3 -m unittest tests/test_14-non-standard-embedded-relationship.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test14"
        cls.FILES = [
            dict(file="non-standard-embedded-relationship.json", ignore_embedded_relationships=False, stix2arango_note="test14"),
        ]


    def test_query_1(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test14_edge_collection
            FILTER doc._is_ref == true
              RETURN doc
        )
        """
        expected_result = [3]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # number of _ref / _refs in bundle

    def test_query_2(self):
        query = """
        FOR doc IN test14_edge_collection
          FILTER doc._is_latest == true
          AND doc._is_ref == true
          AND doc.created_by_ref == "identity--72e906ce-ca1b-5d73-adcd-9ea9eb66a1b4"
            SORT doc.relationship_type
            RETURN DISTINCT doc.relationship_type
        """
        expected_result = [
              "created-by",
              "non-standard"
            ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the type of _ref / _refs in bundle

if __name__ == '__main__':
    unittest.main()
      


import sys
from unittest.mock import patch
import pytest
from stix2arango.__main__ import parse_arguments, main


@pytest.mark.parametrize(
    "args",
    [
        [],
        ["--database", "somedb"],
        ["--file", "somefile"],
        ["--file", "somefile", "--database", "somedb"],
        ["--collection", "somefile", "--database", "somedb"],
    ],
)
def test_database_and_file_required(monkeypatch, args):
    monkeypatch.setattr(sys, "argv", ["prog", *args])
    with pytest.raises(SystemExit) as e:
        parse_arguments()


@pytest.mark.parametrize(
    "args,parsed_output",
    [
        (
            ["--collection", "somefile", "--database", "somedb", "--file", "some-file"],
            {
                "file": "some-file",
                "is_large_file": False,
                "database": "somedb",
                "create_db": True,
                "collection": "somefile",
                "stix2arango_note": "",
                "ignore_embedded_relationships": False,
                "ignore_embedded_relationships_sro": False,
                "ignore_embedded_relationships_smo": False,
                "include_embedded_relationships_attributes": None,
            },
        ),
        (
            [
                "--collection",
                "somefile",
                "--database",
                "somedb",
                "--file",
                "some-file",
                "--ignore_embedded_relationships_smo",
                "yes",
            ],
            {
                "file": "some-file",
                "is_large_file": False,
                "database": "somedb",
                "create_db": True,
                "collection": "somefile",
                "stix2arango_note": "",
                "ignore_embedded_relationships": False,
                "ignore_embedded_relationships_sro": False,
                "ignore_embedded_relationships_smo": True,
                "include_embedded_relationships_attributes": None,
            },
        ),
        (
            [
                "--collection",
                "somefile",
                "--database",
                "somedb",
                "--file",
                "some-file",
                "--ignore_embedded_relationships_smo",
                "yes",
                "--include_embedded_relationships_attributes",
                "abc_ref",
                "abcdef_refs",
            ],
            {
                "file": "some-file",
                "is_large_file": False,
                "database": "somedb",
                "create_db": True,
                "collection": "somefile",
                "stix2arango_note": "",
                "ignore_embedded_relationships": False,
                "ignore_embedded_relationships_sro": False,
                "ignore_embedded_relationships_smo": True,
                "include_embedded_relationships_attributes": ["abc_ref", "abcdef_refs"],
            },
        ),
    ],
)
def test_parse_args(monkeypatch, args, parsed_output):
    args = ["prog", *args]
    monkeypatch.setattr(sys, "argv", args)
    assert parse_arguments().__dict__ == parsed_output


def test_main_calls_correctly(monkeypatch):
    args = [
        "prog",
        "--collection",
        "somefile",
        "--database",
        "somedb",
        "--file",
        "some-file",
        "--ignore_embedded_relationships_smo",
        "yes",
        "--include_embedded_relationships_attributes",
        "abc_ref",
        "abcdef_refs",
    ]
    monkeypatch.setattr(sys, "argv", args)
    with patch("stix2arango.__main__.Stix2Arango") as mock_s2a_init:
        main()
        mock_s2a_init.assert_called_once_with(
            **{
                "file": "some-file",
                "is_large_file": False,
                "database": "somedb",
                "create_db": True,
                "collection": "somefile",
                "stix2arango_note": "",
                "ignore_embedded_relationships": False,
                "ignore_embedded_relationships_sro": False,
                "ignore_embedded_relationships_smo": True,
                "include_embedded_relationships_attributes": ["abc_ref", "abcdef_refs"],
            }
        )





import pytest
from stix2arango import config
from stix2arango.services.arangodb_service import ArangoDBService
from stix2arango.stix2arango.stix2arango import Stix2Arango
import arango.exceptions
from unittest.mock import patch, MagicMock


def test_get_db_name():
    assert ArangoDBService.get_db_name("test") == 'test_database'
    assert ArangoDBService.get_db_name("test_database") == 'test_database'

@pytest.fixture(scope='module')
def service():
    s = ArangoDBService('test_s2a_arango_service', [], [], host_url=Stix2Arango.ARANGODB_URL, username=config.ARANGODB_USERNAME, password=config.ARANGODB_PASSWORD, create_db=True)
    yield s
    s.sys_db.delete_database('test_s2a_arango_service_database')


def test_init_creates_graph(service):
    assert service.db.has_graph('test_s2a_arango_service_graph')


def test_create_db(service):
    service.create_database('test_s2a_arango_service2')
    assert service.sys_db.has_database('test_s2a_arango_service2')
    service.create_database('test_s2a_arango_service2')
    service.sys_db.delete_database('test_s2a_arango_service2')

def test_create_collection(service):
    assert service.create_collection('ss_vertex_collection').info()['name'] == 'ss_vertex_collection', "created collection must have the passed name"
    assert service.create_collection('ss_vertex_collection').info()['name'] == 'ss_vertex_collection', "recreating should not throw error"

def test_execute_raw_query(service):
    c = service.create_collection('execute_raw_query')
    c.insert_many([{"a": 1}, {"a": 2}, {"a": 3}])
    assert service.execute_raw_query("FOR d IN @@collection FILTER d.a >= 2 RETURN d.a", bind_vars={'@collection': c.name}) == [2, 3]
    with pytest.raises(arango.exceptions.AQLQueryExecuteError):
        service.execute_raw_query("FOR d IN @@collection FILTER d.a >= 2 RETURN d.a")

def test_fix_edge_ref():
    assert ArangoDBService.fix_edge_ref('nothing') == "missing_collection/nothing"
    assert ArangoDBService.fix_edge_ref('abcd/nothing') == "abcd/nothing"

def test_transactional_resets_db(service):
    db = service.db
    with patch('arango.database.TransactionDatabase.commit_transaction') as mock_commit, service.transactional():
        pass
    assert service.db == db, "db not reset to original after context exit"
    mock_commit.assert_called_once()


def test_transactional_resets_db__on_failure(service):
    db = service.db
    with patch('arango.database.TransactionDatabase.abort_transaction') as mock_abort, pytest.raises(Exception), service.transactional():
        raise Exception("must raise")
    assert service.db == db, "db not reset to original after context exit"
    mock_abort.assert_called_once()




import argparse
from stix2arango.stix2arango import Stix2Arango


def parse_bool(value: str):
    value = value.lower()
    # ["false", "no", "n"]
    return value in ["yes", "y", "true", "1"]

def parse_ref(value: str):
    if not (value.endswith('_ref') or value.endswith('_refs')):
        raise argparse.ArgumentTypeError('value must end with _ref or _refs')
    return value


def parse_arguments():
    parser = argparse.ArgumentParser(description="Import STIX JSON into ArangoDB")
    parser.add_argument("--file", required=True, help="Path to STIX JSON file")
    parser.add_argument(
        "--is_large_file",
        action="store_true",
        help="Use large file mode [Use this mode when the bundle is very large, this will enable you stix2arango to chunk before loading into memory]",
    )
    parser.add_argument("--database", required=True, help="ArangoDB database name")
    parser.add_argument(
        "--create_db",
        default=True,
        type=parse_bool,
        help="whether or not to skip the creation of database, requires admin permission",
    )
    parser.add_argument("--collection", required=True, help="ArangoDB collection name")
    parser.add_argument(
        "--stix2arango_note", required=False, help="Note for the import", default=""
    )
    parser.add_argument(
        "--ignore_embedded_relationships",
        required=False,
        help="Ignore Embedded Relationship for the import",
        type=parse_bool,
        default=False,
    )
    parser.add_argument(
        "--ignore_embedded_relationships_sro",
        required=False,
        help="Ignore Embedded Relationship for imported SROs",
        type=parse_bool,
        default=False,
    )
    parser.add_argument(
        "--ignore_embedded_relationships_smo",
        required=False,
        help="Ignore Embedded Relationship for imported SMOs",
        type=parse_bool,
        default=False,
    )
    parser.add_argument(
        "--include_embedded_relationships_attributes",
        required=False,
        help="Only create embedded relationships for keys",
        action="extend",
        nargs="+",
        type=parse_ref
    )
    return parser.parse_args()


def main():
    args = parse_arguments()
    stix_obj = Stix2Arango(
        database=args.database,
        collection=args.collection,
        file=args.file,
        create_db=args.create_db,
        stix2arango_note=args.stix2arango_note,
        ignore_embedded_relationships=args.ignore_embedded_relationships,
        ignore_embedded_relationships_sro=args.ignore_embedded_relationships_sro,
        ignore_embedded_relationships_smo=args.ignore_embedded_relationships_smo,
        is_large_file=args.is_large_file,
        include_embedded_relationships_attributes=args.include_embedded_relationships_attributes,
    )
    stix_obj.run()



# python3 -m unittest tests/test_21-embedded-sro-update-for-sco.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test21"
        cls.FILES = [
            dict(file="nested-embedded-ref.json", ignore_embedded_relationships=False, stix2arango_note="test21"),
            dict(file="nested-embedded-ref.json", ignore_embedded_relationships=False, stix2arango_note="test21"),
        ]


    def test_query_1(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test21_edge_collection
            FILTER doc._is_latest == true
            AND doc._is_ref == true
            AND doc._stix2arango_note != "automatically imported on collection creation"
              RETURN doc
        )
        """
        expected_result = [7]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the latest 4 address_ref and 3 object_marking refs latest version

    def test_query_2(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test21_edge_collection
            FILTER doc._is_latest == false
            AND doc._is_ref == true
            AND doc._stix2arango_note != "automatically imported on collection creation"
              RETURN doc
        )
        """
        expected_result = [0]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # no update happens here because the updated SROs for embedded ref have identical properties

    def test_query_3(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test21_edge_collection
            FILTER doc.id == "relationship--e33fa156-4d06-55e1-969e-97b25b68002d"
            AND HAS(doc, 'modified') OR HAS(doc, 'created')
            RETURN doc
        )
        """
        expected_result = [0]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the embedded SRO should not have modified or created time because the source transaction object does not

    def test_query_4(self):
        query = """
        FOR doc IN test21_edge_collection
            FILTER doc.id == "relationship--e33fa156-4d06-55e1-969e-97b25b68002d"
            RETURN {
                modified: doc.modified,
                created: doc.created
            }
        """
        expected_result = [
          {
            "modified": None,
            "created": None
          }
        ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # a follow up to test 3 to check these keys do not exist in object

if __name__ == '__main__':
    unittest.main()


from datetime import datetime
from unittest.mock import MagicMock, patch
import pytest
from stix2arango import utils
from stix2arango.config import DOGESEC_IDENTITY


def test_load_file_from_url():
    assert (
        utils.load_file_from_url(DOGESEC_IDENTITY)["id"]
        == "identity--9779a2db-f98c-5f4b-8d08-8ee04e02dbb5"
    )

    with pytest.raises(Exception):
        utils.load_file_from_url("https://gogle.cm/ojkskja")


def test_remove_duplicates():
    assert (
        len(
            utils.remove_duplicates(
                [
                    {
                        "id": "all-the-same",
                    },
                    {
                        "id": "all-the-same",
                    },
                    {"id": "all-the-same", "a": 1},
                    {"id": "all-the-same", "a": 1, "_is_latest": True},
                    {"id": "all-the-same", "a": 2, "b": 1},
                    {"id": "all-the-same", "a": 2, "b": 1, "c": 1},
                    {"id": "all-the-same", "a": 2, "b": 1, "c": 1, "_is": 8},
                ]
            )
        )
        == 4
    )


def test_get_embedded_refs():
    assert utils.get_embedded_refs(
        {
            "abc_ref": "ref1",
            "abcd_refs": ["ref1", "ref2"],
            "abcde": [{"abcdef_ref": "ref7"}, {"abcd_efgh_ref": "ref8"}],
        }
    ) == [
        ("abc", ["ref1"]),
        ("abcd", ["ref1", "ref2"]),
        ("abcde-abcdef", ["ref7"]),
        ("abcde-abcd-efgh", ["ref8"]),
    ]


def test_get_embedded_refs_empty():
    assert utils.get_embedded_refs(
        {
            "abc_ref": "ref1",
            "empty_ref": "",  # skipped entirely
            "some_empty_refs": ["ref10", "", "ref9"],  # empty ref skipped
            "abcd_refs": ["ref1", "ref2"],
            "abcde": [{"abcdef_ref": "ref7"}, {"abcd_efgh_ref": "ref8"}],
        }
    ) == [
        ("abc", ["ref1"]),
        ("some-empty", ["ref10", "ref9"]),
        ("abcd", ["ref1", "ref2"]),
        ("abcde-abcdef", ["ref7"]),
        ("abcde-abcd-efgh", ["ref8"]),
    ]


def test_get_embedded_refs__attributes_whitelist():
    assert utils.get_embedded_refs(
        {
            "abc_ref": "ref1",
            "abcd_refs": ["ref1", "ref2"],
            "abcde": [{"abcdef_ref": "ref7"}, {"abcd_efgh_ref": "ref8"}],
        },
        attributes=["abcd_efgh_ref", "abc_ref"],
    ) == [
        ("abc", ["ref1"]),
        (
            "abcde-abcd-efgh",
            ["ref8"],
        ),
    ]


def test_get_vertex_and_edge_collection_names():
    assert utils.get_vertex_and_edge_collection_names("ade") == (
        "ade_vertex_collection",
        "ade_edge_collection",
    )
    assert utils.get_vertex_and_edge_collection_names("ade_edge_collection") == (
        "ade_vertex_collection",
        "ade_edge_collection",
    )
    assert utils.get_vertex_and_edge_collection_names("ade_vertex_collection") == (
        "ade_vertex_collection",
        "ade_edge_collection",
    )


def test_create_relationship__no_targets():
    assert utils.create_relationship_obj(None, None, None, None, None, None) == []


def test_create_relationships():
    obj = {
        "created": "2024-01-01T00:00:00.000000",
        "modified": "2024-01-02T00:00:00.000000",
        "object_marking_refs": ["marking--xyz"],
    }

    source = "indicator--source"
    targets = ["malware--target1", "software--target--2"]
    relationship = "indicates"
    insert_statement = []
    bundle_id = "bundle--xyz"
    extra_data = {"confidence": 80, "labels": ["example"]}
    arango_obj = MagicMock()
    arango_obj.identity_ref = {"id": "identity--abcd"}
    arango_obj.core_collection_vertex = "vertices"
    arango_obj.file = "/some/path/file.txt"
    arango_obj.note = "This is a note"

    utils.create_relationship_obj(
        obj=obj,
        source=source,
        targets=targets,
        relationship=relationship,
        insert_statement=insert_statement,
        bundle_id=bundle_id,
        arango_obj=arango_obj,
        extra_data=extra_data,
    )

    assert len(insert_statement) == 2
    assert insert_statement == [
        {
            "relationship_type": "indicates",
            "created": "2024-01-01T00:00:00.000000",
            "modified": "2024-01-02T00:00:00.000000",
            "object_marking_refs": ["marking--xyz"],
            "id": "relationship--8a86537f-d4bb-55bc-b06d-ddeabbd9899d",
            "created_by_ref": "identity--abcd",
            "source_ref": "indicator--source",
            "target_ref": "malware--target1",
            "_from": "vertices/indicator--source",
            "_to": "vertices/malware--target1",
            "_bundle_id": "bundle--xyz",
            "_file_name": "file.txt",
            "_stix2arango_note": "This is a note",
            "_is_ref": True,
            "type": "relationship",
            "spec_version": "2.1",
            "external_references": [
                {
                    "source_name": "stix2arango",
                    "description": "embedded-relationship",
                }
            ],
            "confidence": 80,
            "labels": ["example"],
        },
        {
            "relationship_type": "indicates",
            "created": "2024-01-01T00:00:00.000000",
            "modified": "2024-01-02T00:00:00.000000",
            "object_marking_refs": ["marking--xyz"],
            "id": "relationship--fda071b0-e1e8-5831-a707-850f1b1a5a0e",
            "created_by_ref": "identity--abcd",
            "source_ref": "indicator--source",
            "target_ref": "software--target--2",
            "_from": "vertices/indicator--source",
            "_to": "vertices/software--target--2",
            "_bundle_id": "bundle--xyz",
            "_file_name": "file.txt",
            "_stix2arango_note": "This is a note",
            "_is_ref": True,
            "type": "relationship",
            "spec_version": "2.1",
            "external_references": [
                {
                    "source_name": "stix2arango",
                    "description": "embedded-relationship",
                }
            ],
            "confidence": 80,
            "labels": ["example"],
        },
    ]



# python3 -m unittest tests/test_16-testing-when-sro-source-not-in-collection.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test16"
        cls.FILES = [
            dict(file="source-object-does-not-exist.json", ignore_embedded_relationships=True, stix2arango_note="test16"),
        ]


    def test_query_1(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test16_edge_collection
              RETURN doc
        )
        """
        expected_result = [1]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the one relationship should exist

    def test_query_2(self):
        query = """
        FOR doc IN test16_edge_collection
            FILTER doc.id == "relationship--00038d0e-7fc7-41c3-9055-edb4d87ea912"
            AND doc._stix2arango_ref_err == true
                RETURN {
                _stix2arango_ref_err: doc._stix2arango_ref_err,
                id: doc.id
            }
        """
        expected_result = [
              {
                "_stix2arango_ref_err": True,
                "id": "relationship--00038d0e-7fc7-41c3-9055-edb4d87ea912"
              }
            ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # because the target does not exist, the relationship should show a _stix2arango_ref_err property
                  


# python3 -m unittest tests/test_00-basic-import-logic-with-embedded.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test00"
        
        cls.FILES = [
            dict(file="sigma-rule-bundle.json", ignore_embedded_relationships=False, stix2arango_note="test00")
        ]

    def test_query_1(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test00_vertex_collection
            FILTER doc._is_latest == true
            AND doc._stix2arango_note != "automatically imported on collection creation"
            AND doc._stix2arango_note == "test00"
              RETURN doc
        )
        """
        expected_result = [3107]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # grouping 136 + indicator 2969 + marking definition 1 + identity 1 = 3,107

    def test_query_2(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test00_vertex_collection
            FILTER doc._is_latest == false
            AND doc._stix2arango_note != "automatically imported on collection creation"
            AND doc._stix2arango_note == "test00"
              RETURN doc
        )
        """
        expected_result = [0]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # as no updated versions imported yet

    def test_query_3(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test00_vertex_collection
            FILTER doc._is_latest == false
            AND doc._stix2arango_note == "automatically imported on collection creation"
              RETURN doc
        )
        """
        expected_result = [0]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # because only one version of auto imported objects should ever exist

    def test_query_4(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test00_edge_collection
            FILTER doc._is_latest == true
            AND doc._is_ref == false
            AND doc._stix2arango_note == "test00"
              RETURN doc
        )
        """
        expected_result = [914]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # same number of relationships in the bundle

    def test_query_5(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test00_edge_collection
            FILTER doc._is_latest == false
            AND doc._is_ref == false
            AND doc._stix2arango_note == "test00"
              RETURN doc
        )
        """
        expected_result = [0]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # because no update have happened

    def test_query_6(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test00_edge_collection
            FILTER doc._is_latest == true
            AND doc._is_ref == true
            AND doc.created_by_ref == "identity--72e906ce-ca1b-5d73-adcd-9ea9eb66a1b4"
            AND doc._stix2arango_note == "test00"
              RETURN doc
        )
        """
        expected_result = [15032]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the number of embedded relationships in the bundle

    def test_query_7(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test00_edge_collection
            FILTER doc._is_latest == false
            AND doc._is_ref == true
            AND doc.created_by_ref == "identity--72e906ce-ca1b-5d73-adcd-9ea9eb66a1b4"
            AND doc._stix2arango_note == "test00"
              RETURN doc
        )
        """
        expected_result = [0]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # because no updates have happended yet

    def test_query_8(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test00_edge_collection
            FILTER doc._is_latest == true
            AND doc._is_ref == true
            AND doc._stix2arango_note == "test00"
            AND doc.created_by_ref == "identity--72e906ce-ca1b-5d73-adcd-9ea9eb66a1b4"
            AND doc.relationship_type == "target_ref"
            OR doc.relationship_type == "source_ref"
              RETURN doc
        )
        """
        expected_result = [0]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # because no embedded relationships should ever be generated from the `source_ref` and `target_ref` properties

    def test_query_9(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test00_edge_collection
            FILTER doc._stix2arango_ref_err == true
              RETURN {
                stix2arango_ref_err: doc._stix2arango_ref_err,
                id: doc.id
              }
            )
        """
        expected_result = [167]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

    def test_query_10(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test00_edge_collection
            FILTER doc._stix2arango_ref_err == false
              RETURN {
                stix2arango_ref_err: doc._stix2arango_ref_err,
                id: doc.id
              }
            )
        """
        expected_result = [15779]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

    def test_query_11(self):
        query = """
        FOR doc IN test00_edge_collection
          FILTER doc._is_latest == true
          AND doc._is_ref == true
          AND doc._stix2arango_note == "test00"
          AND doc.created_by_ref == "identity--72e906ce-ca1b-5d73-adcd-9ea9eb66a1b4"
            RETURN DISTINCT doc.relationship_type
        """
        expected_result = {
            "object-marking",
            "created-by",
            "object"
        }
        result = self.query_arango(query)
        self.assertEqual(set(result['result']), expected_result)

        # the unique embedded relationship keys that exist for this bundle

if __name__ == '__main__':
    unittest.main()



# python3 -m unittest tests/test_10-updating-object-no-modified-time.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test10"
        cls.FILES = [
            dict(file="sco-original.json", ignore_embedded_relationships=False, stix2arango_note="test10"),
            dict(file="sco-updated.json", ignore_embedded_relationships=False, stix2arango_note="test10"),
        ]


    def test_query_1(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test10_vertex_collection
                FILTER doc._is_latest == true
                AND doc._stix2arango_note != "automatically imported on collection creation"
                	RETURN doc
        )
        """
        expected_result = [2]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # b/c two software objects in bundle

    def test_query_2(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test10_vertex_collection
                FILTER doc._is_latest == false
                AND doc._stix2arango_note != "automatically imported on collection creation"
                    RETURN doc
        )
        """
        expected_result = [1]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # only 1 of the two software objects is actually updated (only one object has properties changed between bundles)

    def test_query_3(self):
        query = """
        FOR doc IN test10_vertex_collection
            FILTER doc.id == "software--55388d12-8d7d-5ed1-b324-817a293a6854"
            SORT doc._is_latest DESC
            RETURN {
                id: doc.id,
                _is_latest: doc._is_latest,
            }
        """
        expected_result = [
              {
                "id": "software--55388d12-8d7d-5ed1-b324-817a293a6854",
                "_is_latest": True
              },
              {
                "id": "software--55388d12-8d7d-5ed1-b324-817a293a6854",
                "_is_latest": False
              }
            ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # this is the object that was updated

    def test_query_4(self):
        query = """
        FOR doc IN test10_vertex_collection
            FILTER doc.id == "software--6d38c3e0-ea8b-5b83-b370-5407523589a9"
            SORT doc._is_latest DESC
            RETURN {
                id: doc.id,
                _is_latest: doc._is_latest,
            }
        """
        expected_result = [
              {
                "id": "software--6d38c3e0-ea8b-5b83-b370-5407523589a9",
                "_is_latest": True
              }
            ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # this object was not updated, so only one version exists

if __name__ == '__main__':
    unittest.main()



import json
import os
from pathlib import Path
from unittest.mock import MagicMock, patch, call

import pytest
from stix2arango.stix2arango.stix2arango import Stix2Arango


def test_is_large_file():
    s2a = Stix2Arango(
        file="some-file",
        database="s2a_tests",
        collection="test_run",
        is_large_file=True,
    )
    with (
        patch("stix2arango.stix2arango.stix2arango.BundleLoader") as mock_bundle_loader,
        patch.object(Stix2Arango, "run_with_bundle") as mock_run_with_bundle,
    ):
        mocked_bundle = mock_bundle_loader.return_value
        mocked_bundle.chunks = [1, 2, 3]
        mocked_bundle.bundle_id = "my-bundle"
        s2a.run()
        mock_run_with_bundle.assert_has_calls(
            [
                call(dict(type="bundle", objects=x, id="my-bundle"))
                for x in mock_bundle_loader.return_value.chunks
            ]
        )


def test_file_normal():
    data = {"type": "bundle", "id": "some-id", "objects": []}
    f = Path("some-file")
    f.write_text(json.dumps(data))
    s2a = Stix2Arango(
        file=f.name, database="s2a_tests", collection="test_run", is_large_file=False
    )
    with (
        patch("stix2arango.stix2arango.stix2arango.BundleLoader") as mock_bundle_loader,
        patch.object(Stix2Arango, "run_with_bundle") as mock_run_with_bundle,
    ):
        s2a.run()
        mock_bundle_loader.assert_not_called()
        mock_run_with_bundle.assert_called_once_with(data)


def test_add_object_alter_fn():
    s2a = Stix2Arango(
        file='', database="s2a_tests", collection="test_run", is_large_file=False
    )
    with pytest.raises(ValueError):
        s2a.add_object_alter_fn(None)

    with pytest.raises(ValueError):
        s2a.add_object_alter_fn(['some', 'array', 1])

def test_alter_functions():
    s2a = Stix2Arango(
        file='', database="s2a_tests", collection="test_run", is_large_file=False
    )
    fn1 = MagicMock()
    fn2 = MagicMock()
    fn3 = lambda x: 1/0
    bundle = {"type": "bundle", "id": "some-id", "objects": [{'id': 'obj--1', 'type': 'some-type'}, {'id': 'obj--2', 'type': 'some-type'}]}
    s2a.add_object_alter_fn(fn1)
    s2a.add_object_alter_fn(fn2)
    s2a.add_object_alter_fn(fn3)
    s2a.run(data=bundle)
    fn1.assert_has_calls([call(x) for x in bundle['objects']], any_order=True)
    fn2.assert_has_calls([call(x) for x in bundle['objects']], any_order=True)




import sqlite3
import atexit

class VersionManager:
    def __init__(self, db_name, table_name):
        self.db_name = db_name
        self.table_name = table_name
        
        # Connect to the SQLite database
        self.conn = sqlite3.connect(self.db_name)
        self.cursor = self.conn.cursor()
        
        # Create the versions table if it does not exist
        self.create_table()
        atexit.register(self.close)
    
    def create_table(self):
        # Create the table with an additional 'reason' column if it does not exist
        self.cursor.execute(f'''
            CREATE TABLE IF NOT EXISTS {self.table_name} (
                version TEXT PRIMARY KEY,
                failed BOOLEAN DEFAULT 0,
                reason TEXT
            )
        ''')
        self.conn.commit()
    
    def set_failed(self, version, failed=True, reason=None):
        # Insert or update the version in the database, including the reason for failure
        self.cursor.execute(f'''
            INSERT INTO {self.table_name} (version, failed, reason)
            VALUES (?, ?, ?)
            ON CONFLICT(version) 
            DO UPDATE SET failed = ?, reason = ?
        ''', (version, failed, reason, failed, reason))
        self.conn.commit()
    
    def remove_version(self, version):
        # Remove a version from the database once it's done
        self.cursor.execute(f'''
            DELETE FROM {self.table_name}
            WHERE version = ?
        ''', (version,))
        self.conn.commit()

    def get_versions(self, failed=None):
        # Get versions, filter by failed status if provided
        if failed is None:
            self.cursor.execute(f'SELECT version, failed FROM {self.table_name}')
        else:
            self.cursor.execute(f'''
                SELECT version, failed FROM {self.table_name}
                WHERE failed = ?
            ''', (failed,))
        
        return {version: failed for version, failed in self.cursor.fetchall()}

    def close(self):
        # Close the database connection
        print(f'closing {self.db_name}...')
        self.conn.close()

    def recreate_table(self):
        print('recreating tables')
        # Delete the table and recreate it
        self.cursor.execute(f'DROP TABLE IF EXISTS {self.table_name}')
        self.create_table()


# python3 -m unittest tests/test_09-duplicate-objects-but-diff-times-in-bundle.py

import unittest
from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test09"
        cls.FILES = [
            dict(file="duplicate-objects-properties-different.json", ignore_embedded_relationships=False, stix2arango_note="test09"),
        ]


    def test_query_1(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test09_vertex_collection
                FILTER doc._is_latest == true
                AND doc._stix2arango_note != "automatically imported on collection creation"
                    RETURN doc
        )
        """
        expected_result = [2]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

    def test_query_2(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test09_vertex_collection
                FILTER doc._is_latest == false
                AND doc._stix2arango_note != "automatically imported on collection creation"
                    RETURN doc
        )
        """
        expected_result = [1]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

    def test_query_3(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test09_edge_collection
                FILTER doc._is_latest == true
                AND doc._is_ref == false
                AND doc._stix2arango_note != "automatically imported on collection creation"
                    RETURN doc
        )
        """
        expected_result = [1]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

    def test_query_4(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test09_edge_collection
                FILTER doc._is_latest == false
                AND doc._is_ref == false
                AND doc._stix2arango_note != "automatically imported on collection creation"
                    RETURN doc
        )
        """
        expected_result = [1]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

    def test_query_5(self):
        query = """
        FOR doc IN test09_vertex_collection
            FILTER doc.id == "indicator--7a5dedb9-30f9-51c0-a49d-91aeda1fd7fd"
            SORT doc.modified DESC
            RETURN {
                id: doc.id,
                _is_latest: doc._is_latest,
                modified: doc.modified
            }
        """
        expected_result = [
              {
                "id": "indicator--7a5dedb9-30f9-51c0-a49d-91aeda1fd7fd",
                "_is_latest": True,
                "modified": "2024-01-01T00:00:00.000Z"
              },
              {
                "id": "indicator--7a5dedb9-30f9-51c0-a49d-91aeda1fd7fd",
                "_is_latest": False,
                "modified": "2020-10-16T00:00:00.000Z"
              }
            ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

    def test_query_6(self):
        query = """
        FOR doc IN test09_vertex_collection
            FILTER doc.id == "software--50fa0834-9c63-5b0f-bf0e-dce02183253a"
            RETURN {
                id: doc.id,
                _is_latest: doc._is_latest,
            }
        """
        expected_result = [
              {
                "id": "software--50fa0834-9c63-5b0f-bf0e-dce02183253a",
                "_is_latest": True
              }
            ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

if __name__ == '__main__':
    unittest.main()



# python3 -m unittest tests/test_04-update-detected-because-of-stix2arango-note.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test04"

        cls.FILES = [
            dict(file="sigma-rule-bundle.json", ignore_embedded_relationships=False, stix2arango_note="test04A"),
            dict(file="sigma-rule-bundle.json", ignore_embedded_relationships=False, stix2arango_note="test04B"),
            dict(file="sigma-rule-bundle.json", ignore_embedded_relationships=False, stix2arango_note="test04C"),
        ]

    def test_query_1(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test04_vertex_collection
                FILTER doc._is_latest == true
                AND doc._stix2arango_note != "automatically imported on collection creation"
                AND doc._stix2arango_note == "test04C"
                RETURN doc
        )
        """
        expected_result = [3107]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # number of objects in one bundle and b/c 4c considered latest

    def test_query_2(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test04_vertex_collection
                FILTER doc._is_latest == false
                AND doc._stix2arango_note != "automatically imported on collection creation"
                AND (doc._stix2arango_note == "test04A" OR doc._stix2arango_note == "test04B")
                RETURN doc
        )
        """
        expected_result = [6214]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # 3107 * 2 -- number of objects in two bundles and
        # b/c 4a and 4b are considered older versions

    def test_query_3(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test04_vertex_collection
                FILTER doc._is_latest == true
                AND doc._stix2arango_note == "automatically imported on collection creation"
                RETURN doc
        )
        """
        expected_result = [12]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # 9 items from `templates/marking-definition.json`, `marking-definition/stix2arango.json`,
        # `identity/stix2arango.json`, `identity/dogesec.json` = 12

    def test_query_4(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test04_vertex_collection
                FILTER doc._is_latest == false
                AND doc._stix2arango_note == "automatically imported on collection creation"
                RETURN doc
        )
        """
        expected_result = [0]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # because only one version of auto imported objects should ever exist

    def test_query_5(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test04_vertex_collection
                FILTER doc._stix2arango_note != "automatically imported on collection creation"
                AND doc.id == "indicator--d38c3e67-c14b-5d67-84c7-5400fb66d368"
                RETURN doc
        )
        """
        expected_result = [3]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # b/c object is present in each update

    def test_query_6(self):
        query = """
        FOR doc IN test04_vertex_collection
            FILTER doc._stix2arango_note != "automatically imported on collection creation"
            AND doc.id == "indicator--d38c3e67-c14b-5d67-84c7-5400fb66d368"
            SORT doc._record_modified DESC
            RETURN {
                _stix2arango_note: doc._stix2arango_note,
                _is_latest: doc._is_latest,
                id: doc.id
            }
        """
        expected_result = [
            {
                "_stix2arango_note": "test04C",
                "_is_latest": True,
                "id": "indicator--d38c3e67-c14b-5d67-84c7-5400fb66d368"
            },
            {
                "_stix2arango_note": "test04B",
                "_is_latest": False,
                "id": "indicator--d38c3e67-c14b-5d67-84c7-5400fb66d368"
            },
            {
                "_stix2arango_note": "test04A",
                "_is_latest": False,
                "id": "indicator--d38c3e67-c14b-5d67-84c7-5400fb66d368"
            }
        ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the order of updates

    def test_query_7(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test04_edge_collection
                FILTER doc._is_latest == true
                AND doc._is_ref == false
                AND doc._stix2arango_note == "test04C"
                RETURN doc
        )
        """
        expected_result = [914]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # same number of relationships as in the bundle

    def test_query_8(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test04_edge_collection
                FILTER doc._is_latest == false
                AND doc._is_ref == false
                AND (doc._stix2arango_note == "test04A" OR doc._stix2arango_note == "test04B")
                RETURN doc
        )
        """
        expected_result = [1828]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # old relationships x 2 = 914 x 2 = 1828

    def test_query_9(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test04_edge_collection
                FILTER doc._is_ref == false
                AND doc.id == "relationship--3089bdec-3d25-5d1b-a6ac-9d152ab14e35"
                RETURN doc
        )
        """
        expected_result = [3]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # As have been 3 updates

    def test_query_10(self):
        query = """
        FOR doc IN test04_edge_collection
            FILTER doc._is_ref == false
            AND doc.id == "relationship--3089bdec-3d25-5d1b-a6ac-9d152ab14e35"
            SORT doc._record_modified DESC
            RETURN {
                _stix2arango_note: doc._stix2arango_note,
                _is_latest: doc._is_latest,
                id: doc.id
            }
        """
        expected_result = [
            {
                "_stix2arango_note": "test04C",
                "_is_latest": True,
                "id": "relationship--3089bdec-3d25-5d1b-a6ac-9d152ab14e35"
            },
            {
                "_stix2arango_note": "test04B",
                "_is_latest": False,
                "id": "relationship--3089bdec-3d25-5d1b-a6ac-9d152ab14e35"
            },
            {
                "_stix2arango_note": "test04A",
                "_is_latest": False,
                "id": "relationship--3089bdec-3d25-5d1b-a6ac-9d152ab14e35"
            }
        ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the order of updates

    def test_query_11(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test04_edge_collection
                FILTER doc._is_latest == true
                AND doc._is_ref == true
                AND doc.created_by_ref == "identity--72e906ce-ca1b-5d73-adcd-9ea9eb66a1b4"
                AND doc._stix2arango_note == "test04C"
                RETURN doc
        )
        """
        expected_result = [15032]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the number of valid embedded relationships in the bundle

    def test_query_12(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test04_edge_collection
                FILTER doc._is_latest == false
                AND doc._is_ref == true
                AND doc.created_by_ref == "identity--72e906ce-ca1b-5d73-adcd-9ea9eb66a1b4"
                AND (doc._stix2arango_note == "test04A" OR doc._stix2arango_note == "test04B")
                RETURN doc
        )
        """
        expected_result = [30064]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # 15302 x 2 for both old embedded relationships

    def test_query_13(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test04_edge_collection
                FILTER doc._is_ref == true
                AND doc.id == "relationship--5b32a703-4317-5f58-b1ce-03735c756035"
                RETURN doc
        )
        """
        expected_result = [3]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # 3 updates have happened

    def test_query_14(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test04_edge_collection
                FILTER doc._is_ref == true
                AND doc._is_latest == true
                AND doc.id == "relationship--5b32a703-4317-5f58-b1ce-03735c756035"
                RETURN doc
        )
        """
        expected_result = [1]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # but only one should be is latest

    def test_query_15(self):
        query = """
        FOR doc IN test04_edge_collection
            FILTER doc._is_ref == true
            AND doc.id == "relationship--5b32a703-4317-5f58-b1ce-03735c756035"
            SORT doc._record_modified DESC
            RETURN {
                _stix2arango_note: doc._stix2arango_note,
                _is_latest: doc._is_latest,
                id: doc.id
            }
        """
        expected_result = [
              {
                "_stix2arango_note": "test04C",
                "_is_latest": True,
                "id": "relationship--5b32a703-4317-5f58-b1ce-03735c756035"
              },
              {
                "_stix2arango_note": "test04B",
                "_is_latest": False,
                "id": "relationship--5b32a703-4317-5f58-b1ce-03735c756035"
              },
              {
                "_stix2arango_note": "test04A",
                "_is_latest": False,
                "id": "relationship--5b32a703-4317-5f58-b1ce-03735c756035"
              }
            ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the order of updates

if __name__ == '__main__':
    unittest.main()


# python3 -m unittest tests/test_18-testing-nested-embedded-ref.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test18"
        cls.FILES = [
            dict(file="nested-embedded-ref.json", ignore_embedded_relationships=False, stix2arango_note="test18"),
        ]


    def test_query_1(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test18_edge_collection
            FILTER doc._is_ref == true
            AND doc._is_latest == true
              RETURN doc
        )
        """
        expected_result = [7]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # contains 4 address_ref and 3 object_marking refs

    def test_query_2(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test18_edge_collection
                FILTER doc._is_ref == true
                AND doc._is_latest == true
                AND doc.created != null
                AND doc.modified != null
                LET keys = ATTRIBUTES(doc)
                LET filteredKeys = keys[* FILTER !STARTS_WITH(CURRENT, "_")]
                RETURN KEEP(doc, filteredKeys)
        )
        """
        expected_result = [0]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # this checks no created and modified times are generated in embedded SROS (because source transaction object does not have these times).

    def test_query_3(self):
        query = """
        FOR doc IN test18_edge_collection
            FILTER doc._is_ref == true
            AND doc._is_latest == true
            SORT doc.relationship_type
                RETURN DISTINCT doc.relationship_type
        """
        expected_result = [
              "object-marking",
              "output-address",
            ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the distinct embedded ref object relationship types for this search

    def test_query_4(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test18_edge_collection
                FILTER doc.id == "relationship--35f9c60e-5364-556e-a6f0-ccb0179eec02"
                COLLECT modified = doc.modified, created = doc.created
                RETURN { modified, created }
            )
        """
        expected_result = [1]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # checks only one version with created and modified times exist, b/c no update yet

if __name__ == '__main__':
    unittest.main()



import os
import logging
from dotenv import load_dotenv
from uuid import UUID

load_dotenv()

logging.basicConfig(
    level=logging.INFO,
    format="[%(asctime)s] %(levelname)s - %(message)s",  # noqa D100 E501
    datefmt="%Y-%m-%d - %H:%M:%S",
)
ARANGODB_HOST = os.getenv("ARANGODB_HOST")
ARANGODB_PORT = os.getenv("ARANGODB_PORT")
ARANGODB_USERNAME = os.getenv("ARANGODB_USERNAME")
ARANGODB_PASSWORD = os.getenv("ARANGODB_PASSWORD")

json_schema = {
    "type": "object",
    "properties": {
        "type": {"type": "string", "const": "bundle"},
        "id": {"type": "string"},
        "objects": {"type": "array", "items": {"type": "object"}}
    },
    "required": ["type", "id", "objects"]
}
STIX2ARANGO_IDENTITY = "https://github.com/muchdogesec/stix4doge/raw/main/objects/identity/stix2arango.json" # this is stix2arango identity
DOGESEC_IDENTITY = "https://github.com/muchdogesec/stix4doge/raw/main/objects/identity/dogesec.json" # this is stix2arango identity

STIX2ARANGO_MARKING_DEFINITION = "https://raw.githubusercontent.com/muchdogesec/stix4doge/main/objects/marking-definition/stix2arango.json" # this is stix2arango marking-definition

IDENTITY_REFS = [
    STIX2ARANGO_IDENTITY,
    DOGESEC_IDENTITY
]
MARKING_DEFINITION_REFS = [
   STIX2ARANGO_MARKING_DEFINITION
]
DEFAULT_OBJECT_URL = MARKING_DEFINITION_REFS + IDENTITY_REFS

namespace = UUID("72e906ce-ca1b-5d73-adcd-9ea9eb66a1b4")


# python3 -m unittest tests/test_23-embedded-sro-removed.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test23"
        cls.FILES = [
            dict(file="embedded-ref-object.json", ignore_embedded_relationships=False, stix2arango_note="test23"),
            dict(file="embedded-ref-object-removed.json", ignore_embedded_relationships=False, stix2arango_note="test23"),
        ]


# no embedded ref in the object now

    def test_query_1(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test23_edge_collection
            FILTER doc._is_latest == true
            AND doc._is_ref == true
            AND doc._stix2arango_note != "automatically imported on collection creation"
              RETURN doc
        )
        """
        expected_result = [0]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

# original bundle, the object had 3 objects

    def test_query_2(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test23_edge_collection
            FILTER doc._is_latest == false
            AND doc._is_ref == true
            AND doc._stix2arango_note != "automatically imported on collection creation"
              RETURN doc
        )
        """
        expected_result = [3]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

if __name__ == '__main__':
    unittest.main()


import contextlib
from datetime import datetime
import logging
import os
from pathlib import Path
import sqlite3
import tempfile
import uuid
import ijson
import json
from collections import Counter

from stix2arango.utils import get_embedded_refs


class BundleLoader:
    def __init__(self, file_path, chunk_size_min=20_000, db_path=""):
        self.file_path = Path(file_path)
        self.chunk_size_min = chunk_size_min
        self.groups = None
        self.bundle_id = "bundle--" + str(uuid.uuid4())

        self.db_path = db_path
        if not self.db_path:
            self.temp_path = tempfile.NamedTemporaryFile(
                prefix="s2a_bundle_loader--", suffix=".sqlite"
            )
            self.db_path = self.temp_path.name
        self._init_db()

    def _init_db(self):
        """Initialize SQLite DB with objects table."""
        self.conn = sqlite3.connect(self.db_path)
        self.conn.execute(
            """
            CREATE TABLE IF NOT EXISTS objects (
                id TEXT PRIMARY KEY,
                type TEXT,
                raw TEXT
            )
        """
        )
        self.conn.execute("PRAGMA synchronous = OFF;")
        self.conn.execute("PRAGMA journal_mode = MEMORY;")
        self.conn.execute("PRAGMA temp_store = MEMORY;")
        self.conn.commit()

    def save_to_sqlite(self, objects):
        """Save one STIX object to the SQLite database."""
        self.inserted = getattr(self, "inserted", 0)

        try:
            self.conn.executemany(
                "INSERT OR REPLACE INTO objects (id, type, raw) VALUES (?, ?, ?)",
                [(obj["id"], obj["type"], json.dumps(obj)) for obj in objects],
            )
        except sqlite3.IntegrityError as e:
            print(f"Failed to insert len({objects}) objects: {e}")
        else:
            self.conn.commit()
        self.inserted += len(objects)
        # logging.info(f"inserted {self.inserted}")

    @staticmethod
    def get_refs(obj):
        refs = []
        for _type, targets in get_embedded_refs(obj):
            refs.extend(targets)
        return refs

    def build_groups(self):
        """
        Iterates the STIX bundle and uses union-find to group IDs such that for every
        relationship (object of type "relationship"), its own id and its source_ref
        and target_ref end up in the same group.
        """
        all_ids: dict[str, list[str]] = dict()  # All object IDs in the file
        logging.info(f"loading into {self.db_path}")

        with open(self.file_path, "rb") as f:
            objects = ijson.items(f, "objects.item", use_float=True)
            to_insert = []
            for obj in objects:
                obj_id = obj.get("id")
                to_insert.append(obj)
                all_ids.setdefault(obj_id, [])
                if obj["type"] == "relationship" and all(
                    x in obj for x in ["target_ref", "source_ref"]
                ):
                    sr, tr = [obj["source_ref"], obj["target_ref"]]
                    all_ids[obj_id].extend([sr, tr])
                    all_ids.setdefault(sr, []).extend([tr, obj_id])
                    all_ids.setdefault(tr, []).extend([sr, obj_id])
                for ref in self.get_refs(obj):
                    all_ids[obj_id].append(ref)
                if len(to_insert) >= self.chunk_size_min:
                    self.save_to_sqlite(to_insert)
                    to_insert.clear()
            if to_insert:
                self.save_to_sqlite(to_insert)

        logging.info(f"loaded {self.inserted} into {self.db_path}")
        handled = set()

        self.groups = []
        group = set()

        def from_ids(all_ids):
            for obj_id in all_ids:
                if obj_id in handled:
                    continue
                group_objs = {obj_id, *all_ids[obj_id]}
                handled.update(group_objs)
                new_group = group.union(group_objs)
                if len(new_group) >= self.chunk_size_min:
                    group.clear()
                    self.groups.append(tuple(new_group))
                else:
                    group.update(group_objs)

        from_ids(all_ids)
        if group:
            self.groups.append(tuple(group))
        return self.groups

    def load_objects_by_ids(self, ids):
        """Retrieve a list of STIX objects by their IDs from the SQLite database."""
        placeholders = ",".join(["?"] * len(ids))
        query = f"SELECT raw FROM objects WHERE id IN ({placeholders})"
        cursor = self.conn.execute(query, list(ids))
        return [json.loads(row[0]) for row in cursor.fetchall()]

    def get_objects(self, group):
        return list(self.load_objects_by_ids(group))

    @property
    def chunks(self):
        for group in self.groups or self.build_groups():
            yield self.get_objects(group)

    def __del__(self):
        with contextlib.suppress(Exception):
            os.remove(self.db_path)



from collections import defaultdict
import os
import time
from typing import List, Dict
import copy

from datetime import datetime
import os
import typing
from arango.client import ArangoClient
from arango.database import StandardDatabase


def annotate_versions(objects: List[Dict]):
    grouped = defaultdict(list)

    # Group by 'id'
    for obj in objects:
        grouped[obj["id"]].append(obj)

    annotations: list[dict] = []
    deprecated = []

    for obj_id, items in grouped.items():
        # items = [copy.deepcopy(item) for item in items]

        # Separate items with non-None modified
        valid_modified = [item for item in items if item.get("modified") is not None]

        # _is_latest: max(modified) -> max(_record_modified)
        if valid_modified:
            max_modified = max(item.get("modified") for item in valid_modified)
            latest_candidates = [
                item for item in valid_modified if item.get("modified") == max_modified
            ]
            max_record_modified_latest = max(
                item["_record_modified"] for item in latest_candidates
            )
        else:
            max_modified = None
            max_record_modified_latest = max(item["_record_modified"] for item in items)
        # _is_earliest: min(modified) -> max(_record_modified)
        if valid_modified:
            min_modified = min(item.get("modified") for item in valid_modified)
            earliest_candidates = [
                item for item in valid_modified if item.get("modified") == min_modified
            ]
            max_record_modified_earliest = max(
                item["_record_modified"] for item in earliest_candidates
            )
        else:
            min_modified = None
            max_record_modified_earliest = min(
                item["_record_modified"] for item in items
            )

        # _taxii_visible: for each modified (including None), select highest _record_modified
        taxii_visible_keys = set()
        modified_groups = defaultdict(list)
        for item in items:
            modified_groups[item.get("modified")].append(item)

        for mod_val, group in modified_groups.items():
            max_rec_mod = max(i["_record_modified"] for i in group)
            for item in group:
                if item["_record_modified"] == max_rec_mod:
                    taxii_visible_keys.add(item["_key"])

        for item in items:
            is_latest = (
                item.get("modified") == max_modified
                and item["_record_modified"] == max_record_modified_latest
            )

            is_earliest = (
                item.get("modified") == min_modified
                and item["_record_modified"] == max_record_modified_earliest
            )

            if item.get("_is_latest") and not is_latest:
                deprecated.append(item["_id"])
            taxii_ = dict(
                visible=item["_key"] in taxii_visible_keys,
                first=is_earliest,
                last=is_latest,
            )
            if taxii_ == item["_taxii"] and item['_is_latest'] == is_latest:
                continue  # No change
            annotations.append(
                dict(
                    _key=item["_key"],
                    _is_latest=is_latest,
                    _taxii=taxii_,
                )
            )

    return annotations, deprecated



# python3 -m unittest tests/test_12-custom-scos.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test12"
        cls.FILES = [
            dict(file="custom-sco-original.json", ignore_embedded_relationships=False, stix2arango_note="test12"),
            dict(file="custom-sco-updated.json", ignore_embedded_relationships=False, stix2arango_note="test12"),
        ]


    def test_query_1(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test12_vertex_collection
            FILTER doc._is_latest == true
            AND doc._stix2arango_note != "automatically imported on collection creation"
              RETURN doc
        )
        """
        expected_result = [1]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # number of objects in bundle

    def test_query_2(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test12_vertex_collection
            FILTER doc._is_latest == false
            AND doc._stix2arango_note != "automatically imported on collection creation"
              RETURN doc
        )
        """
        expected_result = [1]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # old version of object

    def test_query_3(self):
        query = """
        FOR doc IN test12_vertex_collection
            FILTER doc.id == "custom-sco--0306ac42-a167-4eb6-a67a-969c255a85ae"
            SORT doc._is_latest DESC
            RETURN {
                id: doc.id,
                _is_latest: doc._is_latest,
                name: doc.title
            }
        """
        expected_result = [
              {
                "id": "custom-sco--0306ac42-a167-4eb6-a67a-969c255a85ae",
                "_is_latest": True,
                "name": "new"
              },
              {
                "id": "custom-sco--0306ac42-a167-4eb6-a67a-969c255a85ae",
                "_is_latest": False,
                "name": "old"
              }
            ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # old version of object should be _is_latest false

if __name__ == '__main__':
    unittest.main()



import sys
import time
import unittest
from unittest.mock import patch
import requests
import base64
import json
import subprocess
import os
from dotenv import load_dotenv
from stix2arango.__main__ import main as run_s2a

# Load environment variables from .env file
load_dotenv()

ARANGODB_HOST = os.getenv("ARANGODB_HOST")
ARANGODB_PORT = os.getenv("ARANGODB_PORT")
ARANGODB_USERNAME = os.getenv("ARANGODB_USERNAME")
ARANGODB_PASSWORD = os.getenv("ARANGODB_PASSWORD")

class BaseTestArangoDBQueries(unittest.TestCase):

    script_run = False  # Class variable to ensure the script is run only once

    @classmethod
    def setUpClass(cls):
        cls.load_configuration()
        cls.clear_collections()
        if not cls.script_run:
            cls.run_script()
            cls.script_run = True
        cls.setup_headers_and_url()

    @classmethod
    def load_configuration(cls):
        cls.ARANGODB_DATABASE = os.getenv("ARANGODB_DATABASE")
        cls.ARANGODB_COLLECTION = os.getenv("ARANGODB_COLLECTION")
        cls.FILES: list[dict] = []


        cls.STIX2ARANGO_NOTE_1 = os.getenv("STIX2ARANGO_NOTE_1")
        cls.STIX2ARANGO_NOTE_2 = os.getenv("STIX2ARANGO_NOTE_2")
        cls.STIX2ARANGO_NOTE_3 = os.getenv("STIX2ARANGO_NOTE_3")
        cls.TEST_FILE_1 = os.getenv("TEST_FILE_1")
        cls.TEST_FILE_2 = os.getenv("TEST_FILE_2")
        cls.TEST_FILE_3 = os.getenv("TEST_FILE_3")
        cls.IGNORE_EMBEDDED_RELATIONSHIPS_1 = os.getenv("IGNORE_EMBEDDED_RELATIONSHIPS_1")
        cls.IGNORE_EMBEDDED_RELATIONSHIPS_2 = os.getenv("IGNORE_EMBEDDED_RELATIONSHIPS_2")
        cls.IGNORE_EMBEDDED_RELATIONSHIPS_3 = os.getenv("IGNORE_EMBEDDED_RELATIONSHIPS_3")
        cls.CUSTOM_FLAG_1 = os.getenv("CUSTOM_FLAG_1")
        cls.CUSTOM_FLAG_2 = os.getenv("CUSTOM_FLAG_2")
        cls.CUSTOM_FLAG_3 = os.getenv("CUSTOM_FLAG_3")

    @classmethod
    def clear_collections(cls):
        collections = [cls.ARANGODB_COLLECTION + '_vertex_collection', cls.ARANGODB_COLLECTION + '_edge_collection']
        for collection in collections:
            url = f'http://{ARANGODB_HOST}:{ARANGODB_PORT}/_db/{cls.ARANGODB_DATABASE}_database/_api/collection/{collection}/truncate'
            response = requests.put(url, headers={
                'Authorization': 'Basic ' + base64.b64encode(f'{ARANGODB_USERNAME}:{ARANGODB_PASSWORD}'.encode('utf-8')).decode('utf-8')
            })
            if response.status_code not in [200, 404]:
                raise Exception(f"Failed to clear collection {collection}: {response.text}")

    @classmethod
    def run_script(cls):
        for file in cls.FILES:
            args = ["stix2arango.py"]
            file.update(database=cls.ARANGODB_DATABASE, collection=cls.ARANGODB_COLLECTION, file=f"tests/files/stix2arango/{file['file']}")
            for k,v in file.items():
                args.append("--"+k)
                args.append(str(v))
            with patch.object(sys, 'argv', args):
                setup = run_s2a()
        return

    @classmethod
    def setup_headers_and_url(cls):
        cls.headers = {
            'Content-Type': 'application/json',
            'Authorization': 'Basic ' + base64.b64encode(f'{ARANGODB_USERNAME}:{ARANGODB_PASSWORD}'.encode('utf-8')).decode('utf-8')
        }
        cls.base_url = f'http://{ARANGODB_HOST}:{ARANGODB_PORT}/_db/{cls.ARANGODB_DATABASE}_database/_api/cursor'

    def query_arango(self, query):
        payload = json.dumps({'query': query})
        response = requests.post(self.base_url, headers=self.headers, data=payload)
        response.raise_for_status()
        return response.json()



# python3 -m unittest tests/test_05-update-detected-because-of-modified-time-change.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test05"
        cls.FILES = [
            dict(file="sigma-rule-bundle-condensed-original.json", ignore_embedded_relationships=False, stix2arango_note="test05"),
            dict(file="sigma-rule-bundle-condensed-update-1.json", ignore_embedded_relationships=False, stix2arango_note="test05"),
            dict(file="sigma-rule-bundle-condensed-update-2.json", ignore_embedded_relationships=False, stix2arango_note="test05"),
        ]


    def test_query_1(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test05_vertex_collection
                FILTER doc._is_latest == true
                AND doc._stix2arango_note != "automatically imported on collection creation"
                RETURN doc
        )
        """
        expected_result = [4]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # 4 non-relationship objects in bundle

    def test_query_2(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test05_vertex_collection
                FILTER doc._is_latest == false
                AND doc._stix2arango_note != "automatically imported on collection creation"
                RETURN doc
        )
        """
        expected_result = [2]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # one for each update of `indicator--7a5dedb9-30f9-51c0-a49d-91aeda1fd7fd`

    def test_query_3(self):
        query = """
        FOR doc IN test05_vertex_collection
            FILTER doc._stix2arango_note != "automatically imported on collection creation"
            AND doc.id == "indicator--7a5dedb9-30f9-51c0-a49d-91aeda1fd7fd"
            SORT doc.modified DESC
            RETURN {
                id: doc.id,
                _is_latest: doc._is_latest,
                name: doc.name,
                modified: doc.modified
            }
        """
        expected_result = [
            {
                "id": "indicator--7a5dedb9-30f9-51c0-a49d-91aeda1fd7fd",
                "_is_latest": True,
                "name": "SECOND UPDATE",
                "modified": "2024-01-01T00:00:00.000Z"
            },
            {
                "id": "indicator--7a5dedb9-30f9-51c0-a49d-91aeda1fd7fd",
                "_is_latest": False,
                "name": "FIRST UPDATE",
                "modified": "2023-12-12T00:00:00.000Z"
            },
            {
                "id": "indicator--7a5dedb9-30f9-51c0-a49d-91aeda1fd7fd",
                "_is_latest": False,
                "name": "Deny Service Access Using Security Descriptor Tampering Via Sc.EXE",
                "modified": "2023-02-28T00:00:00.000Z"
            }
        ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the order of insert time with highest modified time as is latest = true

    def test_query_4(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test05_edge_collection
                FILTER doc._is_latest == true
                AND doc._is_ref == false
                RETURN doc
        )
        """
        expected_result = [1]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # only one relationship in bundle

    def test_query_5(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test05_edge_collection
                FILTER doc._is_latest == false
                AND doc._is_ref == false
                RETURN doc
        )
        """
        expected_result = [2]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # as 2 old versions of relationship object

    def test_query_6(self):
        query = """
        FOR doc IN test05_edge_collection
            FILTER doc._stix2arango_note != "automatically imported on collection creation"
            AND doc._is_ref == false
            AND doc.id == "relationship--3089bdec-3d25-5d1b-a6ac-9d152ab14e35"
            SORT doc.modified DESC
                RETURN {
                    id: doc.id,
                    _is_latest: doc._is_latest,
                    modified: doc.modified
                }
        """
        expected_result = [
            {
                "id": "relationship--3089bdec-3d25-5d1b-a6ac-9d152ab14e35",
                "_is_latest": True,
                "modified": "2024-01-01T00:00:00.000Z"
            },
            {
                "id": "relationship--3089bdec-3d25-5d1b-a6ac-9d152ab14e35",
                "_is_latest": False,
                "modified": "2023-12-12T00:00:00.000Z"
            },
            {
                "id": "relationship--3089bdec-3d25-5d1b-a6ac-9d152ab14e35",
                "_is_latest": False,
                "modified": "2023-02-28T00:00:00.000Z"
            }
        ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the order of insert time with highest modified time as is latest = true

    def test_query_7(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test05_edge_collection
                FILTER doc._is_latest == true
                AND doc._is_ref == true
                RETURN doc
        )
        """
        expected_result = [15]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # there 15 valid `_ref` or `_refs` in final bundle

    def test_query_8(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test05_edge_collection
                FILTER doc._is_latest == false
                AND doc._is_ref == true
                RETURN doc
        )
        """
        expected_result = [12]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # only 1 indicator and 1 relationship are ever updated between updates
        # and between them they have 3 embedded relationships each (3x2)x2 = 12

    def test_query_9(self):
        query = """
        FOR doc IN test05_edge_collection
            FILTER doc._is_ref == true
            AND doc.id == "relationship--6b91fcdc-997d-5317-ae08-c001fb6d6d08"
            SORT doc.modified DESC
            RETURN {
                id: doc.id,
                _is_latest: doc._is_latest,
                modified: doc.modified
            }
        """
        expected_result = [
              {
                "id": "relationship--6b91fcdc-997d-5317-ae08-c001fb6d6d08",
                "_is_latest": True,
                "modified": "2023-02-28T00:00:00.000Z"
              }
            ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # because this embedded ref was generated from an objects that is not updated

    def test_query_10(self):
        query = """
        FOR doc IN test05_edge_collection
            FILTER doc._is_ref == true
            AND doc.id == "relationship--fb8916a5-d2b8-5f7e-b381-0c796c354dc3"
            SORT doc.modified DESC
            RETURN {
                id: doc.id,
                _is_latest: doc._is_latest,
                modified: doc.modified
            }
        """
        expected_result = [
              {
                "id": "relationship--fb8916a5-d2b8-5f7e-b381-0c796c354dc3",
                "_is_latest": True,
                "modified": "2024-01-01T00:00:00.000Z"
              },
              {
                "id": "relationship--fb8916a5-d2b8-5f7e-b381-0c796c354dc3",
                "_is_latest": False,
                "modified": "2023-12-12T00:00:00.000Z"
              },
              {
                "id": "relationship--fb8916a5-d2b8-5f7e-b381-0c796c354dc3",
                "_is_latest": False,
                "modified": "2023-02-28T00:00:00.000Z"
              }
            ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # because this embedded ref was generated from an objects that is not updated

if __name__ == '__main__':
    unittest.main()



# python3 -m unittest tests/test_24-test-hidden-properties.py

import unittest
from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test24"
        cls.FILES = [
            dict(file="sigma-rule-bundle.json", ignore_embedded_relationships=False, stix2arango_note="test24"),
        ]

        
# test everything has custom hidden fields in vertex

    def test_query_1(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test24_vertex_collection
            FILTER doc._stix2arango_note != "automatically imported on collection creation"
              AND doc._stix2arango_note != null
              AND doc._record_modified != null 
              AND doc._record_created != null 
              AND doc._is_latest != null
              AND doc._record_md5_hash != null 
              AND doc._bundle_id != null
              AND doc._file_name != null
                RETURN doc
        )
        """
        expected_result = [3107]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

# should match test 1

    def test_query_2(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test24_vertex_collection
            FILTER doc._stix2arango_note != "automatically imported on collection creation"
                RETURN doc
        )
        """
        expected_result = [3107]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

# test everything has custom hidden fields in edge

    def test_query_3(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test24_edge_collection
            FILTER doc._stix2arango_note != null
              AND doc._record_modified != null 
              AND doc._record_created != null 
              AND doc._is_latest != null
              AND doc._record_md5_hash != null 
              AND doc._bundle_id != null 
              AND doc._file_name != null
              AND doc._stix2arango_ref_err != null
              AND doc._record_md5_hash != null 
              AND doc._file_name != null
              AND doc._target_type != null
              AND doc._source_type != null
                RETURN doc
        )
        """
        expected_result = [15946]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

# same as test 3

    def test_query_4(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test24_edge_collection
                RETURN doc
        )
        """
        expected_result = [15946]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

if __name__ == '__main__':
    unittest.main()


# python3 -m unittest tests/test_25-ignore-embedded-rels-smo.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test25"

        cls.FILES = [
            dict(file="embedded-ref-sdo-sco-sro-smo.json", ignore_embedded_relationships=False, stix2arango_note="test25", ignore_embedded_relationships_smo=True),
        ]


## SDO has 3 SCO has 2 SRO has 3 = 8

    def test_query_1(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test25_edge_collection
            FILTER doc._is_latest == true
            AND doc._is_ref == true
            AND doc._stix2arango_note != "automatically imported on collection creation"
              RETURN doc
        )
        """
        expected_result = [8]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the latest object has 3 refs

if __name__ == '__main__':
    unittest.main()



# python3 -m unittest tests/test_01-basic-import-no-embedded.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test01"

        cls.FILES = [
            dict(file="sigma-rule-bundle.json", ignore_embedded_relationships=True, stix2arango_note="test01")
        ]

    def test_query_1(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test01_edge_collection
            FILTER doc._is_latest == true
            AND doc._is_ref == true
              RETURN doc
        )
        """
        expected_result = [0]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # because embedded relationships are set to false

    def test_query_2(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test01_edge_collection
            FILTER doc._is_latest == false
            AND doc._is_ref == true
              RETURN doc
        )
        """
        expected_result = [0]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # because embedded relationships are set to false

if __name__ == '__main__':
    unittest.main()


import argparse
import subprocess
import os
import requests

# Define the version strings
all_versions = [
    "4_9_0"
]

# Sort the versions
all_versions.sort(key=lambda x: list(map(int, x.split('_'))))

def parse_arguments():
    parser = argparse.ArgumentParser(description="Process MITRE ATT&CK versions.")
    parser.add_argument('--versions', type=str, help='Comma-separated list of versions to process (e.g., 1_0,2_0). Default is all versions.')
    parser.add_argument('--ignore_embedded_relationships', action='store_true', help='Flag to ignore embedded relationships. Default is false.')
    parser.add_argument('--database', type=str, default="cti_knowledge_base_store", help='Name of the database to use. Default is "cti".')
    return parser.parse_args()

def create_directory(path):
    if not os.path.exists(path):
        os.makedirs(path)
        print(f"Created directory: {path}")
    else:
        print(f"Directory already exists: {path}")

def download_file(url, destination):
    if os.path.exists(destination):
        print(f"File already exists: {destination}")
        return
    response = requests.get(url)
    if response.status_code == 200:
        with open(destination, 'wb') as file:
            file.write(response.content)
        print(f"Downloaded file to {destination}")
    else:
        print(f"Failed to download file from {url}")

def run_command(command, root_path, ignore_embedded_relationships):
    file_path = os.path.join(root_path, command["file"])
    if not os.path.exists(file_path):
        print(f"File not found: {file_path}")
        return
    try:
        subprocess.run([
            "python3", os.path.join(root_path, "stix2arango.py"),
            "--file", file_path,
            "--database", command["database"],
            "--collection", command["collection"],
            "--stix2arango_note", command.get("stix2arango_note", ""),
            "--ignore_embedded_relationships", str(ignore_embedded_relationships)
        ], check=True)
        print(f"Successfully processed {file_path}")
    except subprocess.CalledProcessError as e:
        print(f"Failed to process {file_path}: {e}")

def main():
    args = parse_arguments()

    if args.versions:
        versions = args.versions.split(',')
    else:
        versions = all_versions

    ignore_embedded_relationships = args.ignore_embedded_relationships
    database = args.database

    # Get the absolute path to the current directory (where this script is located)
    script_path = os.path.dirname(os.path.abspath(__file__))
    root_path = os.path.abspath(os.path.join(script_path, "../.."))  # Move up two levels to the directory containing stix2arango.py and cti_knowledge_base_store
    
    # Define the commands and their arguments for the files
    commands = [
        {
            "file": os.path.join("cti_knowledge_base_store", "mitre-atlas", f"mitre-atlas-v{version}.json"),
            "database": database,
            "collection": "mitre_atlas",
            "stix2arango_note": f"v{version.replace('_', '.')}"
        } for version in versions
    ]
    
    # Collect unique directories to create
    directories_to_create = set()
    for command in commands:
        directory = os.path.dirname(os.path.join(root_path, command["file"]))
        directories_to_create.add(directory)
    
    # Create necessary directories dynamically if they do not already exist
    for directory in directories_to_create:
        create_directory(directory)
    
    # Download files
    base_url = "https://downloads.ctibutler.com/"
    files_to_download = [
        {
            "url": f"{base_url}mitre-atlas-repo-data/mitre-atlas-v{version}.json",
            "destination": os.path.join(root_path, "cti_knowledge_base_store", "mitre-atlas", f"mitre-atlas-v{version}.json")
        } for version in versions
    ]

    for file in files_to_download:
        download_file(file["url"], file["destination"])

    # Run the commands
    for command in commands:
        run_command(command, root_path, ignore_embedded_relationships)

if __name__ == "__main__":
    main()



import os
from dotenv import load_dotenv

import arango
import arango.exceptions
from arango.database import StandardDatabase
from arango import ArangoClient

## run this like `ARANGODB_DATABASE=vulmatch_database python add_ref_types.py` if your database is created by any s2a version < 0.0.4

load_dotenv()

client = ArangoClient(os.getenv('ARANGODB_HOST_URL'), request_timeout=None)
db = client.db(username=os.getenv('ARANGODB_USERNAME'), password=os.getenv('ARANGODB_PASSWORD'), name=os.getenv('ARANGODB_DATABASE'))
for c in db.collections():
    collection_name = c['name']
    if not collection_name.endswith('_edge_collection'):
        continue
    collection = db.collection(collection_name)
    print('updating collection objects with _target_type and _source_type where missing')
    print('this may take a while, depending on collection size')
    print('collection:', collection_name, 'in db:', db.name)
    db.aql.execute("""
        FOR doc IN @@collection
        FILTER doc.type == 'relationship'
        UPDATE {_key: doc._key} WITH {_source_type: FIRST(SPLIT(doc.source_ref, "--")), _target_type: FIRST(SPLIT(doc.target_ref, "--"))} IN @@collection
        """, bind_vars={'@collection': collection_name})
    


# python3 -m unittest tests/test_17-testing-update-of-embedded-refs.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test17"
        cls.FILES = [
            dict(file="embedded-ref-object.json", ignore_embedded_relationships=False, stix2arango_note="test17"),
            dict(file="embedded-ref-object-updated.json", ignore_embedded_relationships=False, stix2arango_note="test17"),
        ]


    def test_query_1(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test17_edge_collection
            FILTER doc._is_ref == true
            AND doc._is_latest == true
              RETURN doc
        )
        """
        expected_result = [4]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # there are 4 embedded refs in the updated version of `indicator--49150a4c-d831-51fa-9f61-aede5570a969`

    def test_query_2(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test17_edge_collection
            FILTER doc._is_ref == true
            AND doc._is_latest == false
              RETURN doc
        )
        """
        expected_result = [3]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # there are 3 embedded refs in the original version of `indicator--49150a4c-d831-51fa-9f61-aede5570a969`

    def test_query_3(self):
        query = """
        FOR doc IN test17_edge_collection
            FILTER doc._is_ref == true
            AND doc._is_latest == true
            SORT doc.relationship_type
                RETURN DISTINCT doc.relationship_type
        """
        expected_result = [
              "created-by",
              "new",
              "object-marking",
            ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the distinct embedded ref objects in `indicator--49150a4c-d831-51fa-9f61-aede5570a969`

    def test_query_4(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test17_edge_collection
                FILTER doc._is_ref == true
                AND doc.id == "relationship--5b32a703-4317-5f58-b1ce-03735c756035"
                COLLECT modified = doc.modified, created = doc.created
                RETURN { modified, created }
            )
        """
        expected_result = [2]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # makes sure that the created and modified times are updated properly

if __name__ == '__main__':
    unittest.main()



# python3 -m unittest tests/test_02-basic-import-no-stix2arango_note.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test02"
        cls.FILES = [
            dict(file="sigma-rule-bundle.json", ignore_embedded_relationships=True)
        ]

    def test_query_1(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test02_vertex_collection
            FILTER doc._is_latest == true
            AND doc._stix2arango_note == ""
              RETURN doc
        )
        """
        expected_result = [3107]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # number of objects in bundle, and stix2arango_note was not entered via CLI

if __name__ == '__main__':
    unittest.main()
    


# python3 -m unittest tests/test_11-custom-sdos.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test11"
        cls.FILES = [
            dict(file="custom-sdo-original.json", ignore_embedded_relationships=False, stix2arango_note="test11"),
            dict(file="custom-sdo-updated.json", ignore_embedded_relationships=False, stix2arango_note="test11"),
        ]


    def test_query_1(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test11_vertex_collection
            FILTER doc._is_latest == true
            AND doc._stix2arango_note != "automatically imported on collection creation"
              RETURN doc
        )
        """
        expected_result = [1]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # number of objects in bundle

    def test_query_2(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test11_vertex_collection
            FILTER doc._is_latest == false
            AND doc._stix2arango_note != "automatically imported on collection creation"
              RETURN doc
        )
        """
        expected_result = [1]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # old version of object

    def test_query_3(self):
        query = """
        FOR doc IN test11_vertex_collection
            FILTER doc.id == "custom-sdo--cbc0b79a-ecbd-59f1-b45b-ea4730df1c2e"
            SORT doc.modified DESC
            RETURN {
                id: doc.id,
                _is_latest: doc._is_latest,
                modified: doc.modified
            }
        """
        expected_result = [
              {
                "id": "custom-sdo--cbc0b79a-ecbd-59f1-b45b-ea4730df1c2e",
                "_is_latest": True,
                "modified": "2024-01-01T00:00:00.000Z"
              },
              {
                "id": "custom-sdo--cbc0b79a-ecbd-59f1-b45b-ea4730df1c2e",
                "_is_latest": False,
                "modified": "2023-10-26T00:00:00.000Z"
              }
            ]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # old version of object

if __name__ == '__main__':
    unittest.main()



# python3 -m unittest tests/test_19_update_with_diff_created_times.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test19"
        cls.FILES = [
            dict(file="update_with_diff_modified_times_1.json", ignore_embedded_relationships=False, stix2arango_note="test19"),
            dict(file="update_with_diff_modified_times_2.json", ignore_embedded_relationships=False, stix2arango_note="test19"),
        ]


    def test_query_1(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test19_vertex_collection
            FILTER doc._is_latest == true
            AND doc._stix2arango_note != "automatically imported on collection creation"
              RETURN doc
        )
        """
        expected_result = [1]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # 1 object is latest

    def test_query_2(self):
        query = """
        RETURN LENGTH(
          FOR doc IN test19_vertex_collection
            FILTER doc._is_latest == false
            AND doc._stix2arango_note != "automatically imported on collection creation"
              RETURN doc
        )
        """
        expected_result = [1]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # 1 object is old

if __name__ == '__main__':
    unittest.main()


import json
import pytest
import sqlite3
import tempfile
from pathlib import Path
from unittest.mock import patch

from stix2arango.stix2arango.bundle_loader import (
    BundleLoader,
)  # adjust this import path

# Sample bundle with related and unrelated objects
STIX_BUNDLE = {
    "type": "bundle",
    "id": "bundle--example",
    "objects": [
        {
            "id": "indicator--1",
            "type": "indicator",
            "bad_ref": "some-ref--7",
            "created_by_ref": "creator--1",
        },
        {
            "id": "indicator--2",
            "type": "indicator",
            "object_marking_refs": ["marking--1", "marking--2"],
        },
        {
            "id": "relationship--1",
            "type": "relationship",
            "source_ref": "indicator--1",
            "target_ref": "indicator--2",
        },
        {"id": "attack-pattern--3", "type": "attack-pattern"},
        {"id": "marking--1", "type": "marking-definition"},
    ],
}


@pytest.fixture
def temp_json_file():
    with tempfile.NamedTemporaryFile(delete=False, mode="w", suffix=".json") as f:
        json.dump(STIX_BUNDLE, f)
        return f.name


def test_init_db_and_tempfile_creation():
    loader = BundleLoader(file_path=tempfile.mkstemp()[1])
    assert isinstance(loader.db_path, str)
    assert Path(loader.db_path).exists()


def test_save_to_sqlite_inserts_objects(temp_json_file):
    loader = BundleLoader(file_path=temp_json_file)
    objects = STIX_BUNDLE["objects"]
    loader.save_to_sqlite(objects)

    cursor = loader.conn.execute("SELECT COUNT(*) FROM objects")
    count = cursor.fetchone()[0]
    assert count == len(objects)


def test_build_groups_creates_correct_groups(temp_json_file):
    loader = BundleLoader(file_path=temp_json_file, chunk_size_min=3)
    groups = loader.build_groups()

    # There should be at least 2 groups: one for related indicators, one for attack-pattern
    assert isinstance(groups, list)
    flat = [id_ for group in groups for id_ in group]
    assert set(flat) == {
        "some-ref--7",
        "indicator--2",
        "indicator--1",
        "relationship--1",
        "creator--1",
        "attack-pattern--3",
        "marking--1",
    }


def test_load_objects_by_ids(temp_json_file):
    loader = BundleLoader(file_path=temp_json_file)
    loader.save_to_sqlite(STIX_BUNDLE["objects"])

    result = loader.load_objects_by_ids(["indicator--1", "attack-pattern--3"])
    assert isinstance(result, list)
    assert {obj["id"] for obj in result} == {"indicator--1", "attack-pattern--3"}


def test_get_objects_returns_objects(temp_json_file):
    loader = BundleLoader(file_path=temp_json_file)
    loader.save_to_sqlite(STIX_BUNDLE["objects"])

    result = loader.get_objects(["indicator--1"])
    assert isinstance(result, list)
    assert result[0]["id"] == "indicator--1"


def test_chunks_generator(temp_json_file):
    loader = BundleLoader(file_path=temp_json_file, chunk_size_min=1)
    chunks = list(loader.chunks)

    assert isinstance(chunks, list)
    for chunk in chunks:
        assert isinstance(chunk, list)
        assert all("id" in obj for obj in chunk)


def test_chunks_with_no_existing_groups_calls_build_groups(temp_json_file):
    loader = BundleLoader(file_path=temp_json_file, chunk_size_min=1)
    with patch.object(loader, "build_groups", wraps=loader.build_groups) as mocked:
        chunks = list(loader.chunks)
        mocked.assert_called_once()
        assert len(chunks) > 0


def test_no_crash_on_missing_relationship_fields(temp_json_file):
    # Remove `source_ref` and `target_ref` to simulate bad data
    bad_bundle = {
        "type": "bundle",
        "objects": [{"id": "bad--1", "type": "relationship"}],
    }
    with tempfile.NamedTemporaryFile(mode="w", delete=False, suffix=".json") as f:
        json.dump(bad_bundle, f)
        path = f.name

    loader = BundleLoader(file_path=path)
    loader.build_groups()
    assert loader.groups  # It should still group the single object


def test_sqlite_file_removed_after_gc(temp_json_file):
    loader = BundleLoader(file_path=temp_json_file, chunk_size_min=1)
    db_path = Path(loader.db_path)
    assert db_path.exists()
    del loader
    assert not db_path.exists(), "should already be deleted"



import argparse
import subprocess
import os
import traceback
import requests
import time
import calendar
from datetime import datetime, timedelta
from stix2arango.stix2arango.stix2arango import Stix2Arango
from manager import VersionManager
from pathlib import Path
from dotenv import load_dotenv

load_dotenv()

manager = VersionManager('utility_fails.db', 'cve_fails')
MAX_RETRIES = 7
# Calculate the latest full year, month, and day
yesterday = datetime.now() - timedelta(days=1)
latest_year = yesterday.year
latest_month = yesterday.month
latest_day = yesterday.day

print(f"Latest full year: {latest_year}, Latest full month: {latest_month}, Latest full day: {latest_day}")

# List of CVE files with daily file names
all_versions = []
for year in range(1988, latest_year + 1):
    start_month = 1
    end_month = 12 if year < latest_year else latest_month
    for month in range(start_month, end_month + 1):
        days_in_month = calendar.monthrange(year, month)[1]
        if year == latest_year and month == latest_month:
            days_in_month = latest_day
        for day in range(1, days_in_month + 1):
            start_date = f"{year}_{str(month).zfill(2)}_{str(day).zfill(2)}"
            version = f"{start_date}-00_00_00-{start_date}-23_59_59"
            all_versions.append((year, month, day, version))

def parse_arguments():
    parser = argparse.ArgumentParser(description="Process NVD CVE versions.")
    parser.add_argument('--min_date', type=str, help='Start date in yyyy-mm-dd format.')
    parser.add_argument('--max_date', type=str, help='End date in yyyy-mm-dd format.')
    parser.add_argument('--ignore_embedded_relationships', action='store_true', help='Flag to ignore embedded relationships. Default is false.')
    parser.add_argument('--database', type=str, default="cti_knowledge_base_store", help='Name of the database to use. Default is "cti_knowledge_base_store".')
    parser.add_argument('--start_over', action='store_true', help='Delete database holding previous attempts.')
    return parser.parse_args()

def filter_versions_by_date(min_date, max_date):
    min_dt = datetime.strptime(min_date, "%Y-%m-%d")
    max_dt = datetime.strptime(max_date, "%Y-%m-%d")
    return [item for item in all_versions if min_dt <= datetime(item[0], item[1], item[2]) <= max_dt]

def create_directory(path):
    if not os.path.exists(path):
        os.makedirs(path)
        print(f"Created directory: {path}")

def download_file(url, destination):
    response = requests.get(url)
    if response.status_code == 200:
        with open(destination, 'wb') as file:
            file.write(response.content)
        print(f"Downloaded file: {destination}")
        return True
    else:
        print(f"Failed to download {url} - Status Code: {response.status_code}")
        return False

def add_cvss_score_to_cve_object(obj):
    if obj['type'] == 'vulnerability':
        x_cvss = list(obj.get('x_cvss', {}).values())
        if not x_cvss:
            return
        primary_cvss = x_cvss[-1]
        for cvss in reversed(x_cvss):
            if cvss['type'].lower() == 'primary':
                primary_cvss = cvss
                break
        if primary_cvss:
            obj['_cvss_base_score'] = primary_cvss['base_score']

def run_command(command, root_path, ignore_embedded_relationships):
    file_path = command["file"]
    if not os.path.exists(file_path):
        print(f"File not found: {file_path}")
        return True
    try:
        print(f"Inserting {file_path} into database {command['database']}...")
        s2a = Stix2Arango(database=command["database"], collection=command["collection"], file=file_path, ignore_embedded_relationships=ignore_embedded_relationships, is_large_file=True, skip_default_indexes=True)
        s2a.add_object_alter_fn(add_cvss_score_to_cve_object)
        s2a.run()
        manager.set_failed(command['version'], failed=False, reason="uploaded to arango")
        print(f"Successfully inserted {command['version']} into database.")
        return True
    except Exception as e:
        print(f"Failed to process {file_path}: {e}")
        manager.set_failed(command['version'], failed=True, reason=str(e))
        return False

def main():
    args = parse_arguments()
    print(f"Database to be used: {args.database}")

    if args.min_date and args.max_date:
        versions = filter_versions_by_date(args.min_date, args.max_date)
    elif args.min_date or args.max_date:
        print("Both --min_date and --max_date must be provided.")
        return
    else:
        versions = all_versions  # If no filter is applied, process all versions.

    if args.start_over:
        print("Recreating database tables...")
        manager.recreate_table()

    new_versions = []
    failed_map = manager.get_versions()
    for item in versions:
        year, month, day, version = item
        if failed_map.get(version) in [True, None]:
            new_versions.append(item)

    print("Will skip previously successful uploads")
    print(f"Only {len(new_versions)} of {len(versions)} items will be processed")
    versions = new_versions

    ignore_embedded_relationships = args.ignore_embedded_relationships
    database = args.database
    
    script_path = os.path.dirname(os.path.abspath(__file__))
    root_path = os.path.abspath(os.path.join(script_path, "../.."))
    
    base_url = "https://cve2stix.vulmatch.com/"
    download_errors = []
    commands = []

    for item in versions:
        year, month, day, version = item
        download_url = f"{base_url}{year}-{str(month).zfill(2)}/cve-bundle-{version}.json"
        destination_path = os.path.join(root_path, "cti_knowledge_base_store", f"nvd-cve/{year}-{str(month).zfill(2)}", f"cve-bundle-{version}.json")

        create_directory(Path(destination_path).parent)
        print(f"Processing: {download_url} -> {destination_path}")
        
        if not os.path.exists(destination_path):
            if not download_file(download_url, destination_path):
                download_errors.append(destination_path)
                continue
        
        commands.append(dict(version=version, collection='nvd_cve', database=database, file=destination_path))

    for command in commands:
        print(f"Running command for version: {command['version']}")
        attempts = 0
        while attempts < MAX_RETRIES:
            try:
                success = run_command(command, root_path, ignore_embedded_relationships)
                if success:
                    break
            except Exception as e:
                print(f"Attempt {attempts+1} failed for {command['version']}: {e}")
            attempts += 1
        if attempts == MAX_RETRIES:
            print(f"upload failed after {attempts} retries, exiting...")
            exit(12)

    if download_errors:
        print("\nDownload Errors:")
        for error in download_errors:
            print(error)

if __name__ == "__main__":
    main()


# python3 -m unittest tests/test_03-import-with-no-update-detected.py

from full_tests.base_test import BaseTestArangoDBQueries

class TestArangoDBQueries(BaseTestArangoDBQueries):

    @classmethod
    def load_configuration(cls):
        super().load_configuration()
        cls.ARANGODB_DATABASE = "s2a_tests"
        cls.ARANGODB_COLLECTION = "test03"

        cls.FILES = [
            dict(file="sigma-rule-bundle.json", ignore_embedded_relationships=False, stix2arango_note="test03"),
            dict(file="sigma-rule-bundle-another.json", ignore_embedded_relationships=False, stix2arango_note="test03"),
            dict(file="sigma-rule-bundle-yet-another.json", ignore_embedded_relationships=False, stix2arango_note="test03"),
        ]

    def test_query_1(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test03_vertex_collection
                FILTER doc._is_latest == true
                AND doc._stix2arango_note != "automatically imported on collection creation"
                AND doc._stix2arango_note == "test03"
                RETURN doc
        )
        """
        expected_result = [3107]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # grouping 136 + indicator 2969 + marking definition 1 + identity 1 = 3,107

    def test_query_2(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test03_vertex_collection
                FILTER doc._is_latest == false
                AND doc._stix2arango_note != "automatically imported on collection creation"
                AND doc._stix2arango_note == "test03"
                RETURN doc
        )
        """
        expected_result = [0]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # as no updates ever happen between versions

    def test_query_3(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test03_edge_collection
                FILTER doc._is_latest == true
                AND doc._is_ref == false
                AND doc._stix2arango_note == "test03"
                RETURN doc
        )
        """
        expected_result = [914]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # same number of relationships in the bundle

    def test_query_4(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test03_edge_collection
                FILTER doc._is_latest == false
                AND doc._is_ref == false
                AND doc._stix2arango_note == "test03"
                RETURN doc
        )
        """
        expected_result = [0]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # because no update should have happened

    def test_query_5(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test03_edge_collection
                FILTER doc._is_latest == true
                AND doc._is_ref == true
                AND doc._stix2arango_note == "test03"
                RETURN doc
        )
        """
        expected_result = [15032]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # the number of embedded relationships in the bundle

    def test_query_6(self):
        query = """
        RETURN LENGTH(
            FOR doc IN test03_edge_collection
                FILTER doc._is_latest == false
                AND doc._is_ref == true
                AND doc._stix2arango_note == "test03"
                RETURN doc
        )
        """
        expected_result = [0]
        result = self.query_arango(query)
        self.assertEqual(result['result'], expected_result)

        # because no update should have happened

if __name__ == '__main__':
    unittest.main()



import pytest
from stix2arango.services.version_annotator import annotate_versions


@pytest.mark.parametrize(
    "objects,expected_annotations,expected_deprecated",
    [
        # Test Case 1: Single object with no versions
        (
            [
                {
                    "id": "indicator--1",
                    "_key": "key1",
                    "_id": "collection/key1",
                    "modified": "2024-01-01T00:00:00.000Z",
                    "_record_modified": "2024-01-01T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                }
            ],
            [
                {
                    "_key": "key1",
                    "_is_latest": True,
                    "_taxii": {"visible": True, "first": True, "last": True},
                }
            ],
            [],
        ),
        # Test Case 1.1: Single object with no versions, _taxii is None
        (
            [
                {
                    "id": "indicator--1",
                    "_key": "key1",
                    "_id": "collection/key1",
                    "modified": "2024-01-01T00:00:00.000Z",
                    "_record_modified": "2024-01-01T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": None,
                }
            ],
            [
                {
                    "_key": "key1",
                    "_is_latest": True,
                    "_taxii": {"visible": True, "first": True, "last": True},
                }
            ],
            [],
        ),
        # Test Case 2: Two versions - simple case
        (
            [
                {
                    "id": "indicator--1",
                    "_key": "key1",
                    "_id": "collection/key1",
                    "modified": "2024-01-01T00:00:00.000Z",
                    "_record_modified": "2024-01-01T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                },
                {
                    "id": "indicator--1",
                    "_key": "key2",
                    "_id": "collection/key2",
                    "modified": "2024-01-02T00:00:00.000Z",
                    "_record_modified": "2024-01-02T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                },
            ],
            [
                {
                    "_key": "key1",
                    "_is_latest": False,
                    "_taxii": {"visible": True, "first": True, "last": False},
                },
                {
                    "_key": "key2",
                    "_is_latest": True,
                    "_taxii": {"visible": True, "first": False, "last": True},
                },
            ],
            [],
        ),
        # Test Case 3: Multiple objects with same modified, different _record_modified
        (
            [
                {
                    "id": "indicator--1",
                    "_key": "key1",
                    "_id": "collection/key1",
                    "modified": "2024-01-01T00:00:00.000Z",
                    "_record_modified": "2024-01-01T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                },
                {
                    "id": "indicator--1",
                    "_key": "key2",
                    "_id": "collection/key2",
                    "modified": "2024-01-01T00:00:00.000Z",
                    "_record_modified": "2024-01-02T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                },
            ],
            [
                # key1 has no changes (already correct), only key2 changes
                {
                    "_key": "key2",
                    "_is_latest": True,
                    "_taxii": {"visible": True, "first": True, "last": True},
                },
            ],
            [],
        ),
        # Test Case 4: Object with None modified field
        (
            [
                {
                    "id": "indicator--1",
                    "_key": "key1",
                    "_id": "collection/key1",
                    "modified": None,
                    "_record_modified": "2024-01-01T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                }
            ],
            [
                {
                    "_key": "key1",
                    "_is_latest": True,
                    "_taxii": {"visible": True, "first": True, "last": True},
                }
            ],
            [],
        ),
        # Test Case 5: Deprecation scenario - object was latest but now isn't
        (
            [
                {
                    "id": "indicator--1",
                    "_key": "key1",
                    "_id": "collection/key1",
                    "modified": "2024-01-01T00:00:00.000Z",
                    "_record_modified": "2024-01-01T00:00:00.000Z",
                    "_is_latest": True,
                    "_taxii": {"visible": True, "first": True, "last": True},
                },
                {
                    "id": "indicator--1",
                    "_key": "key2",
                    "_id": "collection/key2",
                    "modified": "2024-01-02T00:00:00.000Z",
                    "_record_modified": "2024-01-02T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                },
            ],
            [
                {
                    "_key": "key1",
                    "_is_latest": False,
                    "_taxii": {"visible": True, "first": True, "last": False},
                },
                {
                    "_key": "key2",
                    "_is_latest": True,
                    "_taxii": {"visible": True, "first": False, "last": True},
                },
            ],
            ["collection/key1"],
        ),
        # Test Case 6: Multiple objects with different IDs
        (
            [
                {
                    "id": "indicator--1",
                    "_key": "key1",
                    "_id": "collection/key1",
                    "modified": "2024-01-01T00:00:00.000Z",
                    "_record_modified": "2024-01-01T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                },
                {
                    "id": "indicator--2",
                    "_key": "key2",
                    "_id": "collection/key2",
                    "modified": "2024-01-01T00:00:00.000Z",
                    "_record_modified": "2024-01-01T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                },
            ],
            [
                {
                    "_key": "key1",
                    "_is_latest": True,
                    "_taxii": {"visible": True, "first": True, "last": True},
                },
                {
                    "_key": "key2",
                    "_is_latest": True,
                    "_taxii": {"visible": True, "first": True, "last": True},
                },
            ],
            [],
        ),
        # Test Case 7: Complex scenario with multiple versions and modified groups
        (
            [
                {
                    "id": "indicator--1",
                    "_key": "key1",
                    "_id": "collection/key1",
                    "modified": "2024-01-01T00:00:00.000Z",
                    "_record_modified": "2024-01-01T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                },
                {
                    "id": "indicator--1",
                    "_key": "key2",
                    "_id": "collection/key2",
                    "modified": "2024-01-02T00:00:00.000Z",
                    "_record_modified": "2024-01-02T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                },
                {
                    "id": "indicator--1",
                    "_key": "key3",
                    "_id": "collection/key3",
                    "modified": "2024-01-02T00:00:00.000Z",
                    "_record_modified": "2024-01-03T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                },
                {
                    "id": "indicator--1",
                    "_key": "key4",
                    "_id": "collection/key4",
                    "modified": "2024-01-03T00:00:00.000Z",
                    "_record_modified": "2024-01-03T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                },
            ],
            [
                # key1: changes from all False to visible+first
                {
                    "_key": "key1",
                    "_is_latest": False,
                    "_taxii": {"visible": True, "first": True, "last": False},
                },
                # key2: no changes (stays all False)
                # key3: changes from all False to visible
                {
                    "_key": "key3",
                    "_is_latest": False,
                    "_taxii": {"visible": True, "first": False, "last": False},
                },
                # key4: changes from all False to visible+last+_is_latest
                {
                    "_key": "key4",
                    "_is_latest": True,
                    "_taxii": {"visible": True, "first": False, "last": True},
                },
            ],
            [],
        ),
        # Test Case 8: No changes needed (all annotations already correct)
        (
            [
                {
                    "id": "indicator--1",
                    "_key": "key1",
                    "_id": "collection/key1",
                    "modified": "2024-01-01T00:00:00.000Z",
                    "_record_modified": "2024-01-01T00:00:00.000Z",
                    "_is_latest": True,
                    "_taxii": {"visible": True, "first": True, "last": True},
                }
            ],
            [],
            [],
        ),
        # Test Case 9: Mixed None and valid modified fields
        (
            [
                {
                    "id": "indicator--1",
                    "_key": "key1",
                    "_id": "collection/key1",
                    "modified": None,
                    "_record_modified": "2024-01-01T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                },
                {
                    "id": "indicator--1",
                    "_key": "key2",
                    "_id": "collection/key2",
                    "modified": "2024-01-02T00:00:00.000Z",
                    "_record_modified": "2024-01-02T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                },
            ],
            [
                {
                    "_key": "key1",
                    "_is_latest": False,
                    "_taxii": {"visible": True, "first": False, "last": False},
                },
                {
                    "_key": "key2",
                    "_is_latest": True,
                    "_taxii": {"visible": True, "first": True, "last": True},
                },
            ],
            [],
        ),
    ],
    ids=[
        "single_object_no_versions",
        "single_object_no_versions_taxii_none",
        "two_versions_simple",
        "same_modified_different_record",
        "none_modified_field",
        "deprecation_scenario",
        "multiple_different_ids",
        "complex_multiple_versions",
        "no_changes_needed",
        "mixed_none_and_valid_modified",
    ],
)
def test_annotate_versions(objects, expected_annotations, expected_deprecated):
    """Test version annotation with various scenarios"""
    annotations, deprecated = annotate_versions(objects)

    # Sort both lists by _key for consistent comparison
    annotations_sorted = sorted(annotations, key=lambda x: x["_key"])
    expected_sorted = sorted(expected_annotations, key=lambda x: x["_key"])

    assert annotations_sorted == expected_sorted
    assert sorted(deprecated) == sorted(expected_deprecated)


def test_empty_input():
    """Test with empty input list"""
    annotations, deprecated = annotate_versions([])
    assert annotations == []
    assert deprecated == []


@pytest.mark.parametrize(
    "objects,expected_visible_count",
    [
        # One visible per modified value
        (
            [
                {
                    "id": "indicator--1",
                    "_key": "key1",
                    "_id": "collection/key1",
                    "modified": "2024-01-01T00:00:00.000Z",
                    "_record_modified": "2024-01-01T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                },
                {
                    "id": "indicator--1",
                    "_key": "key2",
                    "_id": "collection/key2",
                    "modified": "2024-01-01T00:00:00.000Z",
                    "_record_modified": "2024-01-02T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                },
                {
                    "id": "indicator--1",
                    "_key": "key3",
                    "_id": "collection/key3",
                    "modified": "2024-01-02T00:00:00.000Z",
                    "_record_modified": "2024-01-02T00:00:00.000Z",
                    "_is_latest": False,
                    "_taxii": {"visible": False, "first": False, "last": False},
                },
            ],
            2,  # key2 (max _record_modified for 2024-01-01) and key3
        ),
    ],
    ids=["taxii_visible_per_modified"],
)
def test_taxii_visibility(objects, expected_visible_count):
    """Test that TAXII visibility is correctly assigned"""
    annotations, _ = annotate_versions(objects)
    visible_count = sum(1 for ann in annotations if ann["_taxii"]["visible"])
    assert visible_count == expected_visible_count


def test_multiple_groups_independently_annotated():
    """Test that different object IDs are processed independently"""
    objects = [
        {
            "id": "indicator--1",
            "_key": "key1",
            "_id": "collection/key1",
            "modified": "2024-01-01T00:00:00.000Z",
            "_record_modified": "2024-01-01T00:00:00.000Z",
            "_is_latest": False,
            "_taxii": {"visible": False, "first": False, "last": False},
        },
        {
            "id": "indicator--2",
            "_key": "key2",
            "_id": "collection/key2",
            "modified": "2024-01-01T00:00:00.000Z",
            "_record_modified": "2024-01-01T00:00:00.000Z",
            "_is_latest": False,
            "_taxii": {"visible": False, "first": False, "last": False},
        },
    ]

    annotations, _ = annotate_versions(objects)

    # Both should be latest since they're different objects
    assert all(ann["_is_latest"] for ann in annotations)
    assert len(annotations) == 2
